// Datei BojenA.c
/* ------------------------------------------------------------------------
Fuer praktisch alle Programmiersprachen gilt:
Eine Variable besteht aus einer Adresse und einem Wert.
Einige Variablen haben zusaetzlich einen Namen und/oder einen Zielwert
(somit besteht eine Variable aus 2, 3 oder 4 Teilen).

Eine Konstante besteht aus einem Namen und einem Wert
(d.h. eine Konstante besteht aus genau 2 Teilen).

Den Namen einer Variablen oder Konstanten legt der Programmierer fest.
Die Adresse einer Variablen legt der Ausfuehrer fest.

Haeufig werden unveraenderbare Variablen als Konstanten bezeichnet.
Das ist bedauerlich, weil es dadurch schwierig wird, den (in manchen
Zusammenhaengen wichtigen) konzeptuellen Unterschied zwischen Variablen
und Konstanten zu verstehen:
Eine Variable muss eine Adresse haben, aber nicht unbedingt einen
Namen. Eine Konstante hat keine Adresse (sonst ist sie ein unveraender-
liche Variable), muss aber einen Namen haben.

Variablen und Konstanten kann man grafisch als sog. Bojen darstellen.
In C ist relativ einfach, solche Bojen in Form von "ASCII-Grafiken"
auszugeben. In diesem Programm werden Bojen von Variablen und Konstanten
der Typen int und float ausgegeben.
------------------------------------------------------------------------ */
#include <stdio.h>   // E/A in Bytestroeme, printf, sprintf, ...
#include <stdlib.h>  // atof, atoi, atol, rand, malloc, calloc, free, exit,
                     // abs, div, getenv, system, bsearch, qsort, ...
#include <string.h>  // memcopy, memmove, strcpy, strcat, ...

// ------------------------------------------------------------------------
// Ein paar Unterprogramme zum Ausgeben von Bojen:

void gibIntVarAlsBojeAus  (char * name, int   * adr) {
   // adr sollte die Adresse einer int-Variablen und name ihr Name
   // (oder leer) sein. Die Boje der Variable wird ausgegeben.
   if (strlen(name) > 0) {
      printf("|%s|--<%X>--[%+d]\n", name, adr, *adr);
   } else {
      printf(" %5s  <%X>--[%+d]\n", "",   adr, *adr);
   }
}

void gibFloatVarAlsBojeAus  (char * name, float   * adr) {
   // adr sollte die Adresse einer float-Variablen und name ihr Name
   // (oder leer) sein. Die Boje der Variable wird ausgegeben.
   if (strlen(name) > 0) {
      printf("|%s|--<%X>--[%+6.2f]\n", name, adr, *adr);
   } else {
      printf(" %5s  <%X>--[%+6.2f]\n", "",   adr, *adr);
   }
}

void gibIntKonstAlsBojeAus (char * name, int wert) {
   // wert sollte der Werte eine int-Konstanten und name ihr Name sein.
   // Die Boje der Konstanten wird ausgegeben.
   printf("|%s|--[%+d]\n", name, wert);
}

void gibFloatKonstAlsBojeAus (char * name, float wert) {
   // wert sollte der Werte eine float-Konstanten und name ihr Name sein.
   // Die Boje der Konstanten wird ausgegeben.
   printf("|%s|--[%+6.2f]\n", name, wert);
}
// ------------------------------------------------------------------------
int main() {
   // Ein paar int-Variablen:
   int     iv01 = 123;
   int     iv02 = -17;

   // Ein paar float-Variablen:
   float   fv01 = 12.345;
   float   fv02 = -3.456;

   // Ein paar "int-Konstanten" (gehoeren aber nicht zum Typ int!):
   #define ic01 321
   #define ic02 -71

   // Ein paar "float-Konstanten" (gehoeren aber nicht zum Typ float!):
   #define fc01 543.21
   #define fc02 -6.543

   printf("BojenA: Jetzt geht es los!\n");
   printf("--------------------------\n");
   gibIntVarAlsBojeAus    ("iv01", &iv01);
   gibIntVarAlsBojeAus    ("iv02", &iv02);
   gibIntVarAlsBojeAus    ("", (int *) malloc(sizeof(int)));
   printf("--------------------------\n");
   gibFloatVarAlsBojeAus  ("fv01", &fv01);
   gibFloatVarAlsBojeAus  ("fv02", &fv02);
   gibFloatVarAlsBojeAus  ("", (float *) malloc(sizeof(float)));
   printf("--------------------------\n");
   gibIntKonstAlsBojeAus  ("ic01", ic01);
   gibIntKonstAlsBojeAus  ("ic02", ic02);
   printf("--------------------------\n");
   gibFloatKonstAlsBojeAus("fc01", fc01);
   gibFloatKonstAlsBojeAus("fc02", fc02);
   printf("--------------------------\n");
   printf("BojenA: Das war's erstmal!\n");
   return 0;
} // main
/* ------------------------------------------------------------------------
Ausgabe dieses Programms (Ausfuehrer: Gnu-gcc unter WindowsXP):

BojenA: Jetzt geht es los!
--------------------------
|iv01|--<22CCE4>--[+123]
|iv02|--<22CCE0>--[-17]
        <6605C8>--[+0]                 // Eine namenlose int-Variable
--------------------------
|fv01|--<22CCDC>--[+12.35]
|fv02|--<22CCD8>--[ -3.46]
        <660640>--[ +0.00]             // Eine namenlose float-Variable
--------------------------
|ic01|--[+321]
|ic02|--[-71]
--------------------------
|fc01|--[+543.21]
|fc02|--[ -6.54]
--------------------------
BojenA: Das war's erstmal!

Erlaeuterungen: Hier stellt die erste Boje eine int-Variable dar, die
aus dem Namen iv01, der Adresse 22CCE4 und dem Wert +123 besteht.

Die dritte Boje stellt eine namenlose int-Variable dar, die nur
aus der Adresse 6605C8 und dem Wert +0 besteht.

Es ist ueblich (aber nicht notwendig) Adressen im 16-System (hexadezimal)
darzustellen.

Allgemein werden bei dieser Bojendarstellung
Namen     in senkrechte Striche  |...|
Addressen in spitze Klammern     <...> und
Werte     in eckige Klammern     [...] eingefasst.
---------------------------------------------------------------------------
Ausgabe dieses Programms (Ausfuehrer: Intel icl unter WindowsXP):

BojenA: Jetzt geht es los!
--------------------------
|iv01|--<12FF3C>--[+123]
|iv02|--<12FF40>--[-17]
        <322DC0>--[+3300480]           // Eine namenlose int-Variable
--------------------------
|fv01|--<12FF44>--[+12.35]
|fv02|--<12FF48>--[ -3.46]
        <322DD0>--[ +0.00]             // Eine namenlose float-Variable
--------------------------
|ic01|--[+321]
|ic02|--[-71]
--------------------------
|fc01|--[+543.21]
|fc02|--[ -6.54]
--------------------------
BojenA: Das war's erstmal!
---------------------------------------------------------------------------
Ausgabe dieses Programms (Ausfuehrer: lcc-win32 unter WindowsXP):

BojenA: Jetzt geht es los!
--------------------------
|iv01|--<12FF6C>--[+123]
|iv02|--<12FF68>--[-17]
        <146A80>--[+1311096]           // Eine namenlose int-Variable
--------------------------
|fv01|--<12FF64>--[+12.35]
|fv02|--<12FF60>--[ -3.46]
        <146A90>--[ +0.00]             // Eine namenlose float-Variable
--------------------------
|ic01|--[+321]
|ic02|--[-71]
--------------------------
|fc01|--[+543.21]
|fc02|--[ -6.54]
--------------------------
BojenA: Das war's erstmal!
------------------------------------------------------------------------ */
