// Datei IntTypes02.c
/* ------------------------------------------------------------------------
In diesem Programm werden alle standardisierten Ganzzahltypen (mit Namen
wie int_least8_t, int_fast16_t etc.) benutzt, die jeder mit dem Standard
C99 konforme Ausfuehrer kennen muss.

Ausserdem werden die Makros INT8_C, INT16_C, ..., UINT8_C, UINT16_C, ...,
INTMAX_C und UINTMAX_C dazu benutzt, Literale der standardisierten
Ganzzahltypen zu definieren.
------------------------------------------------------------------------ */
#include <stdint.h>  // int8_t, uint8_t, int_least8_t, uint_least8_t,
                     // int_fast8_t, intptr_t, uintptr_t, intmax_t, ...
#include <inttypes.h>// PRId8, PRIdLEAST8, PRIdFAST8, ...
#include <stdio.h>   // E/A in Bytestroeme, printf, sprintf, ...
#include <limits.h>  // INT_MIN, INT_MAX, LONG_MIN, LONG_MAX, ...

int main() {
   printf("IntTypes02 :  Jetzt geht es los!\n");
   printf("--------------------------------\n");

   // Die Groessen (in Bytes) der standardisierten Typen werden ausgegeben:
   printf("sizeof( int_least8_t ) : %d \n", sizeof( int_least8_t ));
   printf("sizeof( int_least16_t) : %d \n", sizeof( int_least16_t));
   printf("sizeof( int_least32_t) : %d \n", sizeof( int_least32_t));
   printf("sizeof( int_least64_t) : %d \n", sizeof( int_least64_t));
   printf("--------------------------\n");
   printf("sizeof(uint_least8_t ) : %d \n", sizeof(uint_least8_t ));
   printf("sizeof(uint_least16_t) : %d \n", sizeof(uint_least16_t));
   printf("sizeof(uint_least32_t) : %d \n", sizeof(uint_least32_t));
   printf("sizeof(uint_least64_t) : %d \n", sizeof(uint_least64_t));
   printf("--------------------------\n");
   printf("sizeof( int_fast8_t  ) : %d \n", sizeof( int_fast8_t ));
   printf("sizeof( int_fast16_t ) : %d \n", sizeof( int_fast16_t));
   printf("sizeof( int_fast32_t ) : %d \n", sizeof( int_fast32_t));
   printf("sizeof( int_fast64_t ) : %d \n", sizeof( int_fast64_t));
   printf("--------------------------\n");
   printf("sizeof(uint_fast8_t  ) : %d \n", sizeof(uint_fast8_t ));
   printf("sizeof(uint_fast16_t ) : %d \n", sizeof(uint_fast16_t));
   printf("sizeof(uint_fast32_t ) : %d \n", sizeof(uint_fast32_t));
   printf("sizeof(uint_fast64_t ) : %d \n", sizeof(uint_fast64_t));
   printf("--------------------------\n");
   printf("sizeof( intmax_t     ) : %d \n", sizeof( intmax_t    ));
   printf("sizeof(uintmax_t     ) : %d \n", sizeof(uintmax_t    ));
   printf("--------------------------------\n");

   // Variablen der standardisierten Typen werden vereinbart und mit den
   // entsprechenden Makros initialisiert:
    int_least8_t  sil8  =  INT8_C (-123);
    int_least16_t sil16 =  INT16_C(-12345);
    int_least32_t sil32 =  INT32_C(-1234567890);
    int_least64_t sil64 =  INT64_C(-1234567890123456789);

   uint_least8_t  uil8  = UINT8_C ( 123);
   uint_least16_t uil16 = UINT16_C( 12345);
   uint_least32_t uil32 = UINT32_C( 1234567890);
   uint_least64_t uil64 = UINT64_C( 1234567890123456789);

    int_fast8_t   sif8  =  INT8_C (-123);
    int_fast16_t  sif16 =  INT16_C(-12345);
    int_fast32_t  sif32 =  INT32_C(-1234567890);
    int_fast64_t  sif64 =  INT64_C(-1234567890123456789);

   uint_fast8_t   uif8  = UINT8_C ( 123);
   uint_fast16_t  uif16 = UINT16_C( 12345);
   uint_fast32_t  uif32 = UINT32_C( 1234567890);
   uint_fast64_t  uif64 = UINT64_C( 1234567890123456789);

    intmax_t      simax =  INTMAX_C(-1234567890123456789);
   uintmax_t      uimax = UINTMAX_C( 1234567890123456789);

   // Die Namen und Werte der Variablen werden ausgegeben:
   printf("sil8  : %+21"PRId8 "\n", sil8 );
   printf("sil16 : %+21"PRId16"\n", sil16);
   printf("sil32 : %+21"PRId32"\n", sil32);
   printf("sil64 : %+21"PRId64"\n", sil64);
   printf("-----------------------------\n");
   printf("uil8  : %+21"PRId8 "\n", uil8 );
   printf("uil16 : %+21"PRId16"\n", uil16);
   printf("uil32 : %+21"PRId32"\n", uil32);
   printf("uil64 : %+21"PRId64"\n", uil64);
   printf("-----------------------------\n");
   printf("sif8  : %+21"PRId8 "\n", sif8 );
   printf("sif16 : %+21"PRId16"\n", sif16);
   printf("sif32 : %+21"PRId32"\n", sif32);
   printf("sif64 : %+21"PRId64"\n", sif64);
   printf("-----------------------------\n");
   printf("uif8  : %+21"PRId8 "\n", uif8 );
   printf("uif16 : %+21"PRId16"\n", uif16);
   printf("uif32 : %+21"PRId32"\n", uif32);
   printf("uif64 : %+21"PRId64"\n", uif64);
   printf("-----------------------------\n");
   printf("simax : %+21"PRId64"\n", simax);
   printf("uimax : %+21"PRId64"\n", uimax);

   printf("--------------------------------\n");
   printf("IntTypes02 :  Das war's erstmal!\n");
   return 0;
} // main
/* ------------------------------------------------------------------------
Ausgabe:

IntTypes02 :  Jetzt geht es los!
--------------------------------
sizeof( int_least8_t ) : 1
sizeof( int_least16_t) : 2
sizeof( int_least32_t) : 4
sizeof( int_least64_t) : 8
--------------------------
sizeof(uint_least8_t ) : 1
sizeof(uint_least16_t) : 2
sizeof(uint_least32_t) : 4
sizeof(uint_least64_t) : 8
--------------------------
sizeof( int_fast8_t  ) : 1
sizeof( int_fast16_t ) : 4
sizeof( int_fast32_t ) : 4
sizeof( int_fast64_t ) : 8
--------------------------
sizeof(uint_fast8_t  ) : 1
sizeof(uint_fast16_t ) : 4
sizeof(uint_fast32_t ) : 4
sizeof(uint_fast64_t ) : 8
--------------------------
sizeof( intmax_t     ) : 8
sizeof(uintmax_t     ) : 8
--------------------------------
sil8  :                  -123
sil16 :                -12345
sil32 :           -1234567890
sil64 :  -1234567890123456789
-----------------------------
uil8  :                  +123
uil16 :                +12345
uil32 :           +1234567890
uil64 :  +1234567890123456789
-----------------------------
sif8  :                  -123
sif16 :                -12345
sif32 :           -1234567890
sif64 :  -1234567890123456789
-----------------------------
uif8  :                  +123
uif16 :                +12345
uif32 :           +1234567890
uif64 :  +1234567890123456789
-----------------------------
simax :  -1234567890123456789
uimax :  +1234567890123456789
--------------------------------
IntTypes02 :  Das war's erstmal!
------------------------------------------------------------------------ */

