// Datei UngenauInt64Double.c
/* ------------------------------------------------------------------------
Dieses Programm ist eine Erweiterung des Programms UngenauIntFloat.
Machen Sie sich bitte zuerst mit UngenauIntFloat vertraut.

Der C-Standard legt nicht fest, welche Werte zu den Ganzzahltypen
short, int, long, unsigned short, unsigned int etc. gehoeren. Der
Standard C99 schlaegt aber eine Reihe von Namen fuer Ganzzahltypen
vor, deren Eigenschaften festgelegt sind, darunter die Namen
int8_t, int32_t und int64_t.

In diesem Programm wird der Typ int64_t benutzt: Werte dieses Typs
werden mit sscanf eingelesen, mit printf ausgegeben und dazwischen
werden Rechnungen damit durchgefuehrt.

Ansonsten funktioniert dieses Programm ganz entsprechend wie das
Programm UngenauIntFloat (mit int64_t anstelle von int). Es werden
allerdings nur 3 Spalten (1, 3 und 5) ausgegeben, um Platz zu sparen.
------------------------------------------------------------------------ */
#include <stdio.h>   // E/A in Bytestroeme, printf, sprintf, ...
#include <stdbool.h> // Makros bool, true, false.
#include <stdlib.h>  // atof, atoi, atol, rand, malloc, calloc, free, exit,
                     // abs, div, getenv, system, bsearch, qsort, ...
#include <inttypes.h>// PRId8, PRIdLEAST8, PRIdFAST8, ...
// ------------------------------------------------------------------------
// Der Name mit dem dieses Programm gestartet wurde (wird von der
// main-Funktion initialisiert):

char * progName;
// ------------------------------------------------------------------------
int64_t liesInt() {
   // Liest einen int64_t-Wert ein und liefert ihn als Ergebnis. Wenn der
   // Benutzer q anstelle eines int64_t-Wertes eingibt, wird das Programm
   // beendet.
   char eingabe[100];
   int64_t  ergebnis = 0;
   printf("-----------------------------------------------------------\n");
   printf("Eine Ganzzahl (q zum Beenden): ");
   scanf("%s", eingabe);             // Die Eingabe als String einlesen
   if (eingabe[0] == 'q') {
      printf("%s beendet sich!\n", progName);
      exit(0);
   }
   sscanf(eingabe, "%"SCNd64, &ergebnis); // Den String in einen int64_t-Wert
                                          // umwandeln.
   return ergebnis;
} // liesInt
// ------------------------------------------------------------------------
int main(int argc, char * args[]) {
   progName = args[0]; // Der Name, mit dem dieses Programm gestartet wurde.

   // Fuer die folgenden 15 Variablen gilt:
   // d1 ist gleich (float) i1, z1 ist  gleich (int64_t) d1. Fuer die uebrigen
   // Variablen entsprechend.Dabei soll "i" an int64_t, "f" an float und " z" an
   // "hin und zurueck" erinnern.
   // i3 ("in der Mitte zwischen i1 und i5) wird eingelesen.
   // i2 ist die kleinste Zahl unterhalb von i3, die auf denselben
   // float-Wert abgebildet wird wie i3.
   // i4 ist die groesste Zahl oberhalb von i3, die auf denselben
   // float-Wert abgebildet wird wie i3.
   // i1 ist die groesste Zahl unterhalb von i3, die auf einen anderen
   // float-Wert abgebildet wird als i3.
   // i5 ist die kleinste Zahl oberhalb von i3, die auf einen anderen
   // float-Wert abgebildet wird als i3.
   // Insgesamt gilt: i1 <= i2 <= i3 <= i4 <= i5.
   int64_t   i1, i2, i3, i4, i5;
   double    d1, d2, d3, d4, d5;
   int64_t   z1, z2, z3, z4, z5;

   printf("UngenauInt64Double: Jetzt geht es los!\n");
   printf("-----------------------------------------------------------\n");
   printf("Falls Sie mit den Ein- und Ausgaben dieses Programms noch  \n");
   printf("nicht vertraut sind, sollten Sie zuerst das Programm       \n");
   printf("UngenauIntFloat ausprobieren.                              \n");
   printf("\n");
   printf("Interessante Eingaben fuer das vorliegende Programm sind   \n");
   printf("z. B. 12345678901234567 oder 9210987654321098765           \n");

   while (true) {

      // i3 wird eingelesen, d3 und z3 werden daraus berechnet:
      i3 = liesInt();
      d3 = (double)   i3;
      z3 = (int64_t) d3;

      // Die Zahlen i1 und i2, die groesste (bzw. kleinste) Zahl unterhalb
      // von i3, die auf einen anderen (bzw. denselben) double-Wert abge-
      // bildet wird als (bzw. wie) i3 werden berechnet.
      i1 = i3;
      while (true) {
         i1--;
         double d = (double) i1;
         if (d != d3) break;
      }
      i2 = i1+1;

      // Die Zahlen i5 und i4, die kleinste (bzw. groesste) Zahl oberhalb
      // von i3, die auf einen anderen (bzw. denselben) double-Wert abge-
      // bildet wird als (bzw. wie) i3, werden berechnet.
      i5 = i3;
      while (true) {
         i5++;
         double d = (double) i5;
         if (d != d3) break;
      }
      i4 = i5-1;

      d1 = (double) i1;      z1 = (int64_t) d1;
      d2 = (double) i2;      z2 = (int64_t) d2;
//    d3 = (double) i3;      z3 = (int64_t) d3; // Wurde oben schon erledigt
      d4 = (double) i4;      z4 = (int64_t) d4;
      d5 = (double) i5;      z5 = (int64_t) d5;

      // Um die nachfolgenden printf-Befehle uebersichtlicher notieren zu
      // koennen, werden die Umwandlungsbefehle ("Formatsequenzen") mit
      // #define als String-Konstanten definiert:

      #define F1 "%20"PRId64
      #define F2 "%20.0lf"

      printf(F1"=i1, "F1"=i3, "F1"=i5\n", i1, i3, i5);
      printf(F2"=d1, "F2"=d3, "F2"=d5\n", d1, d3, d5);
      printf(F1"=z1, "F1"=z3, "F1"=z5\n", z1, z3, z5);

      // Falls i3 nicht der einzige int64_t-Wert ist, der auf den double-Wert d3
      // abgebildet wird, werden jetzt all diese int64_t-Werte beschrieben:
      int diff = (int) ((i4-i2) + 1);
      if (diff > 1) {
         printf("\n");
         printf("Alle %12d int64_t-Werte               \n",          diff);
         printf("                       zwischen  "F1" \n",            i2);
         printf("                            und  "F1" \n",            i4);
         printf("werden auf denselben double-Wert "F2" abgebildet!\n", d3);
      }
   } // while (true)

   printf("UngenauInt64Double: Das war's erstmal!\n");
} // main
/* ------------------------------------------------------------------------
Ein Dialog mit diesem Programm:

UngenauInt64Double: Jetzt geht es los!
-----------------------------------------------------------
Falls Sie mit den Ein- und Ausgaben dieses Programms noch
nicht vertraut sind, sollten Sie zuerst das Programm
UngenauIntFloat ausprobieren.

Interessante Eingaben fuer das vorliegende Programm sind
z. B. 12345678901234567 oder 9210987654321098765
-----------------------------------------------------------
Eine Ganzzahl (q zum Beenden): 12345678901234567
   12345678901234566=i1,    12345678901234567=i3,    12345678901234570=i5
   12345678901234566=d1,    12345678901234568=d3,    12345678901234570=d5
   12345678901234566=z1,    12345678901234568=z3,    12345678901234570=z5

Alle            3 int64_t-Werte
                       zwischen     12345678901234567
                            und     12345678901234569
werden auf denselben double-Wert    12345678901234568 abgebildet!
-----------------------------------------------------------
Eine Ganzzahl (q zum Beenden): 9210987654321098765
 9210987654321098239=i1,  9210987654321098765=i3,  9210987654321099265=i5
 9210987654321097728=d1,  9210987654321098752=d3,  9210987654321099776=d5
 9210987654321097728=z1,  9210987654321098752=z3,  9210987654321099776=z5

Alle         1025 int64_t-Werte
                       zwischen   9210987654321098240
                            und   9210987654321099264
werden auf denselben double-Wert  9210987654321098752 abgebildet!
-----------------------------------------------------------
Eine Ganzzahl (q zum Beenden): q
UngenauInt64Double beendet sich!
------------------------------------------------------------------------ */
