// Datei UngenauInt64Float.c
/* ------------------------------------------------------------------------
Dieses Programm ist eine Erweiterung des Programms UngenauIntFloat.
Machen Sie sich bitte zuerst mit UngenauIntFloat vertraut.

Der C-Standard legt nicht fest, welche Werte zu den Ganzzahltypen
short, int, long, unsigned short, unsigned int etc. gehoeren. Der
Standard C99 schlaegt aber eine Reihe von Namen fuer Ganzzahltypen
vor, deren Eigenschaften festgelegt sind, darunter die Namen
int8_t, int32_t und int64_t.

In diesem Programm wird der Typ int64_t benutzt: Werte dieses Typs
werden mit sscanf eingelesen, mit printf ausgegeben und dazwischen
werden Rechnungen damit durchgefuehrt.

Ansonsten funktioniert dieses Programm ganz entsprechend wie das
Programm UngenauIntFloat (mit int64_t anstelle von int). Es werden
allerdings nur 3 Spalten (1, 3 und 5) ausgegeben, um Platz zu sparen.
------------------------------------------------------------------------ */
#include <stdio.h>   // E/A in Bytestroeme, printf, sprintf, ...
#include <stdbool.h> // Makros bool, true, false.
#include <stdlib.h>  // atof, atoi, atol, rand, malloc, calloc, free, exit,
                     // abs, div, getenv, system, bsearch, qsort, ...
#include <inttypes.h>// PRId8, PRIdLEAST8, PRIdFAST8, ...
// ------------------------------------------------------------------------
// Der Name mit dem dieses Programm gestartet wurde (wird von der
// main-Funktion initialisiert):

char * progName;
// ------------------------------------------------------------------------
int64_t liesInt() {
   // Liest einen int64_t-Wert ein und liefert ihn als Ergebnis. Wenn der
   // Benutzer q anstelle eines int64_t-Wertes eingibt, wird das Programm
   // beendet.
   char eingabe[100];
   int64_t  ergebnis = 0;
   printf("-----------------------------------------------------------\n");
   printf("Eine Ganzzahl (q zum Beenden): ");
   scanf("%s", eingabe);             // Die Eingabe als String einlesen
   if (eingabe[0] == 'q') {
      printf("%s beendet sich!\n", progName);
      exit(0);
   }
   sscanf(eingabe, "%"SCNd64, &ergebnis); // Den String in einen int64_t-Wert
                                          // umwandeln.
   return ergebnis;
} // liesInt
// ------------------------------------------------------------------------
int main(int argc, char * args[]) {
   progName = args[0]; // Der Name, mit dem dieses Programm gestartet wurde.

   // Fuer die folgenden 15 Variablen gilt:
   // f1 ist gleich (float) i1, z1 ist  gleich (int64_t) f1. Fuer die uebrigen
   // Variablen entsprechend.Dabei soll "i" an int64_t, "f" an float und " z" an
   // "hin und zurueck" erinnern.
   // i3 ("in der Mitte zwischen i1 und i5) wird eingelesen.
   // i2 ist die kleinste Zahl unterhalb von i3, die auf denselben
   // float-Wert abgebildet wird wie i3.
   // i4 ist die groesste Zahl oberhalb von i3, die auf denselben
   // float-Wert abgebildet wird wie i3.
   // i1 ist die groesste Zahl unterhalb von i3, die auf einen anderen
   // float-Wert abgebildet wird als i3.
   // i5 ist die kleinste Zahl oberhalb von i3, die auf einen anderen
   // float-Wert abgebildet wird als i3.
   // Insgesamt gilt: i1 <= i2 <= i3 <= i4 <= i5.
   int64_t   i1, i2, i3, i4, i5;
   float     f1, f2, f3, f4, f5;
   int64_t   z1, z2, z3, z4, z5;

   printf("UngenauInt64Float: Jetzt geht es los!\n");
   printf("-----------------------------------------------------------\n");
   printf("Falls Sie mit den Ein- und Ausgaben dieses Programms noch  \n");
   printf("nicht vertraut sind, sollten Sie zuerst das Programm       \n");
   printf("UngenauIntFloat ausprobieren.                              \n");
   printf("\n");
   printf("Interessante Eingaben fuer das vorliegende Programm sind   \n");
   printf("z. B. 1234567890 oder 123456789012345 oder 1234567890123456\n");
   printf("\n");
   printf("Warnung: Bei noch groesseren Eingaben kann das Berechnen   \n");
   printf("der Ausgaben ziemlich lange dauern!                        \n");

   while (true) {

      // i3 wird eingelesen, f3 und z3 werden daraus berechnet:
      i3 = liesInt();
      f3 = (float)   i3;
      z3 = (int64_t) f3;

      // Die Zahlen i1 und i2, die groesste (bzw. kleinste) Zahl unterhalb
      // von i3, die auf einen anderen (bzw. denselben) float-Wert abge-
      // bildet wird als (bzw. wie) i3 werden berechnet.
      i1 = i3;
      while (true) {
         i1--;
         float f = (float) i1;
         if (f != f3) break;
      }
      i2 = i1+1;

      // Die Zahlen i5 und i4, die kleinste (bzw. groesste) Zahl oberhalb
      // von i3, die auf einen anderen (bzw. denselben) float-Wert abge-
      // bildet wird als (bzw. wie) i3, werden berechnet.
      i5 = i3;
      while (true) {
         i5++;
         float f = (float) i5;
         if (f != f3) break;
      }
      i4 = i5-1;

      f1 = (float) i1;      z1 = (int64_t) f1;
      f2 = (float) i2;      z2 = (int64_t) f2;
//    f3 = (float) i3;      z3 = (int64_t) f3; // Wurde oben schon erledigt
      f4 = (float) i4;      z4 = (int64_t) f4;
      f5 = (float) i5;      z5 = (int64_t) f5;

      // Um die nachfolgenden printf-Befehle uebersichtlicher notieren zu
      // koennen, werden die Umwandlungsbefehle ("Formatsequenzen") mit
      // #define als String-Konstanten definiert:

      #define F1 "%20"PRId64
      #define F2 "%20.0f"

      printf(F1"=i1, "F1"=i3, "F1"=i5\n", i1, i3, i5);
      printf(F2"=f1, "F2"=f3, "F2"=f5\n", f1, f3, f5);
      printf(F1"=z1, "F1"=z3, "F1"=z5\n", z1, z3, z5);

      // Falls i3 nicht der einzige int64_t-Wert ist, der auf den float-Wert f3
      // abgebildet wird, werden jetzt all diese int64_t-Werte beschrieben:
      int diff = (int) ((i4-i2) + 1);
      if (diff > 1) {
         printf("\n");
         printf("Alle %12d int64_t-Werte              \n",          diff);
         printf("                       zwischen "F1" \n",            i2);
         printf("                            und "F1" \n",            i4);
         printf("werden auf denselben float-Wert "F2" abgebildet!\n", f3);
      }
   } // while (true)

   printf("UngenauInt64Float: Das war's erstmal!\n");
} // main
/* ------------------------------------------------------------------------
Ein Dialog mit diesem Programm:

UngenauInt64Float: Jetzt geht es los!
-----------------------------------------------------------
Falls Sie mit den Ein- und Ausgaben dieses Programms noch
nicht vertraut sind, sollten Sie zuerst das Programm
UngenauIntFloat ausprobieren.

Interessante Eingaben fuer das vorliegende Programm sind
z. B. 1234567890 oder 123456789012345 oder 1234567890123456

Warnung: Bei noch groesseren Eingaben kann das Berechnen
der Ausgaben ziemlich lange dauern!
-----------------------------------------------------------
Eine Ganzzahl (q zum Beenden): 1234567890
          1234567871=i1,           1234567890=i3,           1234568001=i5
          1234567808=f1,           1234567936=f3,           1234568064=f5
          1234567808=z1,           1234567936=z3,           1234568064=z5

Alle          129 int64_t-Werte
                       zwischen           1234567872
                            und           1234568000
werden auf denselben float-Wert           1234567936 abgebildet!
-----------------------------------------------------------
Eine Ganzzahl (q zum Beenden): 123456789012345
     123456783908863=i1,      123456789012345=i3,      123456792297473=i5
     123456779714560=f1,      123456788103168=f3,      123456796491776=f5
     123456779714560=z1,      123456788103168=z3,      123456796491776=z5

Alle      8388609 int64_t-Werte
                       zwischen      123456783908864
                            und      123456792297472
werden auf denselben float-Wert      123456788103168 abgebildet!
-----------------------------------------------------------
Eine Ganzzahl (q zum Beenden): 1234567890123456
    1234567881031679=i1,     1234567890123456=i3,     1234568015249409=i5
    1234567813922816=f1,     1234567948140544=f3,     1234568082358272=f5
    1234567813922816=z1,     1234567948140544=z3,     1234568082358272=z5

Alle    134217729 int64_t-Werte
                       zwischen     1234567881031680
                            und     1234568015249408
werden auf denselben float-Wert     1234567948140544 abgebildet!
-----------------------------------------------------------
Eine Ganzzahl (q zum Beenden): q
UngenauInt64Float beendet sich!
------------------------------------------------------------------------ */
