// Datei UngenauIntFloat.c
/* ------------------------------------------------------------------------
Die Sprache C legt die Wertebereiche ihrer Grundtypen (wie int und
float etc.) nicht fest. Fuer den C-Ausfuehrer gcc gilt aber konkret:
Jeder int-Wert und jeder  float-Wert belegt 32 Bit.
Somit gibt es gleich viele int-Werte wie float-Werte (naemlich 2 hoch 32
viele, das sind etwa 4.2 Milliarden). Da float-Werte aus einem wesentlich
groesseren Zahlenbereich stammen (etwa 1.4E-45 bis 3.4E38) als int-Werte
(genau -2 147 483 648 bis +2 147 483 647) kann es nicht fuer jeden
int-Wert einen genau entsprechenden float-Wert geben. Tatsaechlich gilt
im Berecih groesserer int-Werte: Mehrere int-Werte werden durch den
Cast-Befehl (float) auf denselben float-Wert abgebildet.
Anders ausgedrueckt: Fuer viele groessere int-Werte n gilt: n ist
ungleich (int)(float) n. Oder: Die Folge der Ganzzahlen, die man durch
float-Werte exakt darstellen kann, hat Luecken Diese Luecken werden um
so groesser, je weiter man sich von der 0.0 entfernt.

Mit diesem Programm kann man sich diese Tatsache veranschaulichen.
------------------------------------------------------------------------ */
#include <stdio.h>   // E/A in Bytestroeme, printf, sprintf, ...
#include <stdbool.h> // Makros bool, true, false.
#include <stdlib.h>  // atof, atoi, atol, rand, malloc, calloc, free, exit,
                     // abs, div, getenv, system, bsearch, qsort, ...
// ------------------------------------------------------------------------
// Der Name mit dem dieses Programm gestartet wurde (wird von der
// main-Funktion initialisiert):

char * progName;
// ------------------------------------------------------------------------
int liesInt() {
   // Liest einen int-Wert ein und liefert ihn als Ergebnis. Wenn der
   // Benutzer q anstelle eines int-Wertes eingibt, wird das Programm
   // beendet.
   char eingabe[100];
   int  ergebnis = 0;
   printf("-----------------------------------------------------------\n");
   printf("Eine Ganzzahl (q zum Beenden): ");
   scanf("%s", eingabe);             // Die Eingabe als String einlesen
   if (eingabe[0] == 'q') {
      printf("%s beendet sich!\n", progName);
      exit(0);
   }
   sscanf(eingabe, "%i", &ergebnis); // Den String in einen int-Wert
                                     // umwandeln.
   return ergebnis;
} // liesInt
// ------------------------------------------------------------------------
int main(int argc, char * args[]) {
   progName = args[0]; // Der Name, mit dem dieses Programm gestartet wurde.

   // Fuer die folgenden 15 Variablen gilt:
   // f1 ist gleich (float) i1, z1 ist  gleich (int) f1. Fuer die uebrigen
   // Variablen entsprechend.Dabei soll "i" an int, "f" an float und " z" an
   // "hin und zurueck" erinnern.
   // i3 ("in der Mitte zwischen i1 und i5) wird eingelesen.
   // i2 ist die kleinste Zahl unterhalb von i3, die auf denselben
   // float-Wert abgebildet wird wie i3.
   // i4 ist die groesste Zahl oberhalb von i3, die auf denselben
   // float-Wert abgebildet wird wie i3.
   // i1 ist die groesste Zahl unterhalb von i3, die auf einen anderen
   // float-Wert abgebildet wird als i3.
   // i5 ist die kleinste Zahl oberhalb von i3, die auf einen anderen
   // float-Wert abgebildet wird als i3.
   // Insgesamt gilt: i1 <= i2 <= i3 <= i4 <= i5.
   int   i1, i2, i3, i4, i5;
   float f1, f2, f3, f4, f5;
   int   z1, z2, z3, z4, z5;

   printf("UngenauIntFloat: Jetzt geht es los!\n");
   printf("-----------------------------------------------------------\n");
   printf("int   i3 = ...         // Soll von Ihnen eingegeben werden!\n");
   printf("                       // Aus i3 werden dann weitere Zahlen\n");
   printf("                       // f3, z3, i2, i4 etc. berechnet:   \n");
   printf("float f3 = (float) i3; // und                              \n");
   printf("int   z3 = (int)   f3; // ('z' wie 'zuerueck beim Typ int')\n");
   printf("\n");
   printf("i2 ist der kleinste int-Wert, der durch den Cast-Befehl    \n");
   printf("(float) auch auf f3 abgebildet wird (wie i3).              \n");
   printf("i4 ist der groesste int-Wert, der durch den Cast-Befehl    \n");
   printf("(float) auch auf f3 abgebildet wird (wie i3).              \n");
   printf("\n");
   printf("Geben Sie doch mal 2 mit 6, 7 oder 9 Nullen dahinter ein.  \n");

   while (true) {

      // i3 wird eingelesen, f3 und z3 werden daraus berechnet:
      i3 = liesInt();
      f3 = (float) i3;
      z3 = (int)   f3;

      // Die Zahlen i1 und i2, die groesste (bzw. kleinste) Zahl unterhalb
      // von i3, die auf einen anderen (bzw. denselben) float-Wert abge-
      // bildet wird als (bzw. wie) i3 werden berechnet.
      i1 = i3;
      while (true) {
         i1--;
         float f = (float) i1;
         if (f != f3) break;
      }
      i2 = i1+1;

      // Die Zahlen i5 und i4, die kleinste (bzw. groesste) Zahl oberhalb
      // von i3, die auf einen anderen (bzw. denselben) float-Wert abge-
      // bildet wird als (bzw. wie) i3, werden berechnet.
      i5 = i3;
      while (true) {
         i5++;
         float f = (float) i5;
         if (f != f3) break;
      }
      i4 = i5-1;

      f1 = (float) i1;      z1 = (int) f1;
      f2 = (float) i2;      z2 = (int) f2;
//    f3 = (float) i3;      z3 = (int) f3; // Wurde oben schon erledigt
      f4 = (float) i4;      z4 = (int) f4;
      f5 = (float) i5;      z5 = (int) f5;

      // Um die nachfolgenden printf-Befehle uebersichtlicher notieren zu
      // koennen, werden die Umwandlungsbefehle ("Formatsequenzen") mit
      // #define als String-Konstanten definiert:

      #define F1 "%11d"
      #define F2 "%11.0f"

      printf(F1"=i1, "F1"=i2, "F1"=i3, "F1"=i4, "F1"=i5\n", i1, i2, i3, i4, i5);
      printf(F2"=f1, "F2"=f2, "F2"=f3, "F2"=f4, "F2"=f5\n", f1, f2, f3, f4, f5);
      printf(F1"=z1, "F1"=z2, "F1"=z3, "F1"=z4, "F1"=z5\n", z1, z2, z3, z4, z5);

      // Falls i3 nicht der einzige int-Wert ist, der auf den float-Wert f3
      // abgebildet wird, werden jetzt all diese int-Werte beschrieben:
      int diff = (i4-i2) + 1;
      if (diff > 1) {
         printf("\n");
         printf("    Alle %3d int-Werte zwischen "F1" \n",      diff, i2);
         printf("                            und "F1" \n",            i4);
         printf("werden auf denselben float-Wert "F2" abgebildet!\n", f3);
      }
   } // while (true)

   printf("UngenauIntFloat: Das war's erstmal!\n");
} // main
/* ------------------------------------------------------------------------
Ein Dialog mit diesem Programm:

UngenauIntFloat: Jetzt geht es los!
-----------------------------------------------------------
int   i3 = ...         // Soll von Ihnen eingegeben werden!
                       // Aus i3 werden dann weitere Zahlen
                       // f3, z3, i2, i4 etc. berechnet:
float f3 = (float) i3; // und
int   z3 = (int)   f3; // ('z' wie 'zuerueck beim Typ int')

i2 ist der kleinste int-Wert, der durch den Cast-Befehl
(float) auch auf f3 abgebildet wird (wie i3).
i4 ist der groesste int-Wert, der durch den Cast-Befehl
(float) auch auf f3 abgebildet wird (wie i3).

Geben Sie doch mal 2 mit 6, 7 oder 9 Nullen dahinter ein.
-----------------------------------------------------------
Eine Ganzzahl (q zum Beenden): 2000000
    1999999=i1,     2000000=i2,     2000000=i3,     2000000=i4,     2000001=i5
    1999999=f1,     2000000=f2,     2000000=f3,     2000000=f4,     2000001=f5
    1999999=z1,     2000000=z2,     2000000=z3,     2000000=z4,     2000001=z5
-----------------------------------------------------------
Eine Ganzzahl (q zum Beenden): 20000000
   19999998=i1,    19999999=i2,    20000000=i3,    20000001=i4,    20000002=i5
   19999998=f1,    20000000=f2,    20000000=f3,    20000000=f4,    20000002=f5
   19999998=z1,    20000000=z2,    20000000=z3,    20000000=z4,    20000002=z5

    Alle   3 int-Werte zwischen    19999999
                            und    20000001
werden auf denselben float-Wert    20000000 abgebildet!
-----------------------------------------------------------
Eine Ganzzahl (q zum Beenden): 2000000000
 1999999935=i1,  1999999936=i2,  2000000000=i3,  2000000064=i4,  2000000065=i5
 1999999872=f1,  2000000000=f2,  2000000000=f3,  2000000000=f4,  2000000128=f5
 1999999872=z1,  2000000000=z2,  2000000000=z3,  2000000000=z4,  2000000128=z5

    Alle 129 int-Werte zwischen  1999999936
                            und  2000000064
werden auf denselben float-Wert  2000000000 abgebildet!
-----------------------------------------------------------
Eine Ganzzahl (q zum Beenden): q
UngenauIntFloat beendet sich!
------------------------------------------------------------------------ */
