// WahrheitDouble.c
/* ------------------------------------------------------------------------
Das Double-Literal 0.1 bezeichnet nicht etwa den Wert 0.1 ("ein Zehntel"),
sondern einen etwas groesseren Wert. hnliches gilt auch fuer viele andere
Double-Literale.

Diesem Programm kann man Double-Literale (wie z.B. 0.1 oder 123.4567 oder
54.3E2 oder 1.0E-17 etc.) eingeben und bekommt dann den "wahren Wert" des
Literals auf dem Bildschirm angezeigt.

Dieses Programm verwendet die Gnu Multiple Precision Bibliothek libgmp.a.
Ist diese Bibliothek richtig installiert, kann man dieses Programm mit dem
folgenden Kommando uebersetzen und binden lassen:

> gcc WahrheitDouble.c -lgmp -o WahrheitDouble.exe

Falls man die GMP-Bibliothek "nicht installiert hat" aber weiss, dass
die Kopfdatei         gmp.h     im Verzeichnis /K1/K2 und
die Bibliotheksdatei  libgmp.a  im Verzeichnis /B1/B2 liegt,
kann man dieses Programm mit dem folgenden Kommando compilieren und binden:

> gcc -I/K1/K2 WahrheitDouble.c -L/B1/B2 -lgmp -o WahrheitDouble.exe
---------------------------------------------------------------------------
Mit den Programmen WahrheitDouble.c und WahrheitFloat.c kann man zeigen:
Das double-Literal 0.1 und das float-Literal 0.1F haben beide einen etwas
groesseren Wert als 0.1, aber der Wert des double-Literals ist ein bisschen
kleiner als der Wert des float-Literals.
------------------------------------------------------------------------ */
#include <stdio.h>   // E/A in Bytestroeme, printf, scanf, sprintf, ...
                     // fopen, fclose, fflush, fgets, fputs, fread, fwrite, ...
#include <gmp.h>
#include <stdlib.h>  // atof, atoi, atol, rand, malloc, calloc, free, exit,
                     // abs, div, getenv, system, bsearch, qsort, ...
#include <stdbool.h> // Makros bool, true, false.
// ------------------------------------------------------------------------
// Der Name mit dem dieses Programm gestartet wurde (wird von der
// main-Funktion initialisiert):

char * progName;
// ------------------------------------------------------------------------
void gibAusLineal(int einrueckung, int len) {
   // Um das Lesen von Zahlen mit vielen Stellen zu erleichtern, wird nach
   // jeder langen Bruchzahl ein zweizeiliges "Lineal" ausgegeben, von dem
   // man die Spalten-Nr ablesen kann.

   if (len < 10) return; // Kurze Zahlen kann man ohne Lineal lesen

   int i;

   // 1. Lineal-Zeile:
   for (i=0; i<einrueckung; i++) printf(" ");
   i=len;
   while (i>=10) {
      printf("         |");
      i -= 10;
   }
   printf("\n");

   // 2. Lineal-Zeile:
   for (i=0; i<einrueckung; i++) printf(" ");
   i=len;
   int zehner = 10;
   while (i >= 10) {
      printf("       %3d", zehner);
      i      -= 10;
      zehner += 10;
   }
   printf("\n");

} // gibAusLineal
// ------------------------------------------------------------------------
double liesPositiveDouble() {
   // Liest eine Zahl ein, wandelt sie in einen double-Wert um und
   // liefert diesen Wert. Als Eingabe sind alle Zeichenketten erlaubt,
   // die wie ein double-Literal in einem C-Programm aussehen.
   char   eingabe[100];
   double ergebnis = 0.0;
   printf("-----------------------------------------------------------\n");
   printf("Eine Zahl (q zum Beenden): ");
   scanf("%s", eingabe);
   if (eingabe[0] == 'q') {
      printf("%s beendet sich!\n", progName);
      exit(0);
   }
   sscanf(eingabe, "%lf", &ergebnis); // %lf fuer double (%f ist fuer float)!

   // Wir wollen keine negabtiven Zahlen haben:
   if (ergebnis<0.0) {
      printf("Das negative Vorzeichen wird ignoriert!");
      ergebnis = -ergebnis;
   }

   return ergebnis;
} // liesPositiveDouble
// ------------------------------------------------------------------------
void gibAusGmpFloat(mpf_t flt) {
   // Gibt die GMP-float-Zahl flt in (einigermassen) lesbarer Form aus
   // (evtl. mit einem Lineal darunter).

   char     mantisse[100];
   mp_exp_t exponent;
   char *   txt = "Wahrer Wert:";

   // flt wird in einen String von Dezimalziffern (mantisse) und einen
   // Ganzzahlwert (exponent) umgewandelt:
   mpf_get_str(mantisse, &exponent, 10, 0, flt);
   // mantisse  In diesen Speicherbereich werden die Vorpunktstellen der
   //           Zahl flt geschrieben. Allerdings: Wenn flt den Wert 0.0
   //           hat, wird nicht der String "0", sondern ein leerer String
   //           nach mantisse geschrieben.
   // &exponent Zu dieser Adresse wird der Exponent von flt geschrieben
   // 10        Die Mantisse und der Exponent werden im 10-er-System
   //           (Dezimalsystem) dargestellt.
   // 0         Es sollen ALLE Nachpunktstellen von flt erzeugt werden
   //           (haetten wir statt 0 z.B. 10 angegeben, wuerden maximal
   //           10 Nachpunktstellen erzeugt)
   // flt       Diese spezielle GMP-float-Zahl wird in 2 Strings (exponent
   //           und mantisse) umgewandelt.

   printf("\n");
   // Wenn flt den Wert 0.0 hat ist mantisse ein leerer String, aber
   // wir geben "0.0" aus (nicht "0.")
   if (*mantisse == 0) {
      printf("%s 0.0\n", txt);            // flt hat den Wert 0.0
   } else {
      printf("%s 0.%s", txt, mantisse);
      // Der exponent wird nicht ausgegeben wenn er gleich 0 ist:
      if (exponent != 0) {
         printf(" mal 10^%d\n", exponent);
      } else {
         printf("\n");
      }
   } // if-else

   gibAusLineal(strlen(txt)+3, strlen(mantisse));
} // gibAusGmpFloat
// ------------------------------------------------------------------------
void anleitung() {
   printf("-----------------------------------------------------------\n");
   printf("%s",
   "Beispiele fuer Zahlen, die man diesem Programm eingeben darf:\n"
   "\n"
   "  1.5           0      0E3      1.5E3         1e+308\n"
   "  0.1         123    123E+5     0.1E+5        1e-323\n"
   "987.654321  91827  91827E-17  987.654321E-17  1e-324\n"
   "\n"
   "Ihre Eingabe wird wie ein double-Literal interpretiert.\n"
   "Der wahre Wert des Literals wird ausgegeben.\n");
} // anleitung
// ------------------------------------------------------------------------
int main(int argc, char * args[]) {
   progName = args[0];
   printf("%s: Jetzt geht es los!\n", progName);
   anleitung();

   while (true) {
      double dd = liesPositiveDouble();

   // mpf_set_default_prec(200); // Standardgenauigkeit ab jetzt 200 Bits
      mpf_t flt;                 // Eine gmp-float-Variable flt
   // mpf_init(flt);             // Mit Standardgenauigkeit  initialisieren
      mpf_init2(flt, 200);       // Mit Genauigkeit 200 Bits initialisieren
      mpf_set_d(flt, dd);        // Einen double-Wert an flt zuweisen

      gibAusGmpFloat(flt);
   } // while (true)
   printf("-----------------------------------------------------------\n");
   printf("%s: Das war's erstmal!\n", progName);
   return 0;
} // main
/* ------------------------------------------------------------------------
Ein Dialog mit diesem Programm:

WahrheitDouble: Jetzt geht es los!
-----------------------------------------------------------
Beispiele fuer Zahlen, die man diesem Programm eingeben darf:

  1.5           0      0E3      1.5E3         1e+308
  0.1         123    123E+5     0.1E+5        1e-323
987.654321  91827  91827E-17  987.654321E-17  1e-324

Ihre Eingabe wird wie ein double-Literal interpretiert.
Der wahre Wert des Literals wird ausgegeben.
-----------------------------------------------------------
Eine Zahl (q zum Beenden): 0.5

Wahrer Wert: 0.5
-----------------------------------------------------------
Eine Zahl (q zum Beenden): 1234.5

Wahrer Wert: 0.12345 mal 10^4
-----------------------------------------------------------
Eine Zahl (q zum Beenden): 0.1

Wahrer Wert: 0.1000000000000000055511151231257827021181583404541015625
                        |         |         |         |         |
                       10        20        30        40        50
-----------------------------------------------------------
Eine Zahl (q zum Beenden): 1.2345

Wahrer Wert: 0.12344999999999999307220832633902318775653839111328125 mal 10^1
                        |         |         |         |         |
                       10        20        30        40        50
-----------------------------------------------------------
Eine Zahl (q zum Beenden): 1e-9

Wahrer Wert: 0.100000000000000006228159145777985641889706869278597878292202949523926 mal 10^-8
                        |         |         |         |         |         |
                       10        20        30        40        50        60
-----------------------------------------------------------
Eine Zahl (q zum Beenden): q
WahrheitDouble beendet sich!
------------------------------------------------------------------------ */
