// Datei printf01.c
/* ------------------------------------------------------------------------
Formatsequenzen fuer printf-Anweisungen, eine Uebersicht

+--------+--------------+-------------------+-------+-----------------+
|Format  | sinnvolle    | Typ des zu forma- | Vari- | Erlaeu-         |
|sequenz:| Zusaetze:    | tierenden Params: | ante: | terung:         |
+--------+--------------+-------------------+-------+-----------------+
|  %d    | - +   0 blnk | int               | %i    | Ganzzahlen      |
|  %hd   | - +   0 blnk | short             | %hi   | dezimal         |
|  %ld   | - +   0 blnk | long              | %li   | (10-er-System)  |
+--------+--------------+-------------------+-------+-----------------+
|  %u    | -     0      | unsigned int      |       | Ganzzahlen      |
|  %hu   | -     0      | unsigned short    |       | dezimal         |
|  %lu   | -     0      | unsigned long     |       | (10-er-System)  |
+--------+--------------+-------------------+-------+-----------------+
|  %o    | -   # 0      | unsigned int      |       | Ganzzahlen      |
|  %ho   | -   # 0      | unsigned short    |       | oktal           |
|  %lo   | -   # 0      | unsigned long     |       | (8-er-System)   |
+--------+--------------+-------------------+-------+-----------------+
|  %x    | -   # 0      | unsigned int      | %X    | Ganzzahlen      |
|  %hx   | -   # 0      | unsigned short    | %hX   | hexadzimal      |
|  %lx   | -   # 0      | unsigned long     | %lX   | (16-er-System)  |
+--------+--------------+-------------------+-------+-----------------+
|  %f    | - + # 0 blnk | double, float     |       | Bruchzahlen     |
|  %lf   | - + # 0 blnk | double            |       | dezimal         |
|  %Lf   | - + # 0 blnk | long double       |       | ohne Exponent   |
+--------+--------------+-------------------+-------+-----------------+
|  %e    | - + # 0 blnk | double, float     | %E    | Bruchzahlen     |
|  %le   | - + # 0 blnk | double            | %lE   | dezimal         |
|  %Le   | - + # 0 blnk | long double       | %LE   | mit Exponent    |
+--------+--------------+-------------------+-------+-----------------+
|  %g    | - + # 0 blnk | double, float     | %G    | Bruchzahlen     |
|  %lg   | - + # 0 blnk | double            | %lG   | dezimal         |
|  %Lg   | - + # 0 blnk | long double       | %LG   | wie e oder f    |
+--------+--------------+-------------------+-------+-----------------+
|  %a    | - + # 0 blnk | double, float     | %A    | Bruchzahlen     |
|  %la   | - + # 0 blnk | double            | %lA   | hexadezimal     |
|  %La   | - + # 0 blnk | long double       | %LA   |                 |
+--------+--------------+-------------------+-------+-----------------+
|  %c    | -            | int, char         |       |  8-Bit Zeichen  |
|  %lc   | -            | wint_t, wchar_t   |       | 16-Bit Zeichen  |
+--------+--------------+-------------------+-------+-----------------+
|  %s    | -            | char *            |       | String (char)   |
|  %ls   | -            | wchar_t *         |       | String (wchar_t)|
+--------+--------------+-------------------+-------+-----------------+
|  %p    |              | void *            |       | Adresse         |
+--------+--------------+-------------------+-------+-----------------+
|  %n    |              | int *             |       | keine Ausgabe   |
|  %hn   |              | short *           |       |                 |
|  %ln   |              | long *            |       |                 |
+--------+--------------+-------------------+-------+-----------------+

+---------------------------------------------------------------------+
|Sinnvolle Zusaetze (nach dem Prozentzeichen %):                      |
+----+----------------------------------------------------------------+
| -  | Die Ausgabe wird linksbuendig ausgerichtet                     |
|    | (nur mit Mindestbreite sinnvoll, siehe unten)                  |
| +  | Immer Vorzeichen (- oder +    )                                |
|blnk| Immer Vorzeichen (- oder Blank)                                |
| 0  | Mit Nullen auffuellen (statt mit Blanks)                       |
|    | (nur mit Mindestbreite sinnvoll, siehe unten)                  |
| #  | Bei o oder O: Vor der Zahl erscheint 0                         |
| #  | Bei x oder X: Vor der Zahl erscheint x0 bzw. X0                |
| #  | Bei f, F, e, E, g, G und Genauigkeit .0 (siehe unten):         |
|    | Der Punkt erscheint (ohne Stellen danach, z.B. 3.)             |
|    | Ohne # erscheint der Punkt bei Genauigkeit .0 nicht.           |
+----+----------------------------------------------------------------+

+---------------------------------------------------------------------+
|    Unterschied zwischen kleinen Buchstaben x, e, g, a               |
|                     und grossen Buchstaben X, E, G, A               |
|in einer Formatsequenz (z.B. %x oder %X, %e oder %E etc.)            |
+---------------------------------------------------------------------+
|In der Ausgabe erscheinen kleine Buchstaben a, b, c, d, e, f         |
|                     bzw. grosse Buchstaben A, B, C, D, E, F         |
+---------------------------------------------------------------------+
| Zwischen %d und %i sollte es keinen Unterschied geben.              |
+---------------------------------------------------------------------+

+---------------------------------------------------------------------+
| Mindestbreite: Bei allen Formatsequenzen sinnvoll, z.B.             |
| %8d, %5hd, %20ld, %5u, %10ho, %12lx, %15f, %8lg etc.                |
| Legt fest, wieviele Zeichen fuer diese Formatsequenz mindestens     |
| ausgegeben werden. wenn noetig wird das "normale Ergebnis" vorn     |
| oder hinten mit Blanks oder Nullen verlngert (siehe oben           |
| "Sinnvolle Zusaetze" Minus - und Null 0). "Zu lange" Ergebnisse     |
| werden NICHT gekuerzt.                                              |
+---------------------------------------------------------------------+
| Genauigkeit: Allein oder mit Mindestbreite angebbar, z.B.           |
|  %.5d,   %.10f,   %-+.8e (nur Genauigkeit),                         |
| %8.5d, %15.10f, %-+10.8e (Mindestbreite und Genauigkeit)            |
|                                                                     |
| Bedeutung der Genauigkeit:                                          |
| Bei d, i, o, u, x, X: Die Mindestanzahl von Ziffern, die ausgegeben |
|          werden (falls noetig wird vorn mit Nullen verlaengert)     |
| Bei e, E, f: Anzahl von Ziffern nach dem Dezimalpunkt               |
|          ("Nachkommstellen")                                        |
| Bei g, G: Hoeschstzahl der signifikanten Ziffern (d.h.              |
|          Anzahl Vorpunktziffern plus Anzahl Nachpunktziffer)        |
|          "Zu kurze" Ergebnisse werden NICHT verlaengert.            |
| Bei s:   Z.B. %5s: Nur die ersten 5 Zeichen des char * Parameters   |
|          werden genommen (falls er so lang ist).                    |
|          Z.B. %8.5s: Nur die ersten 5 Zeichen (soweit vorhanden)    |
|          werden genommen und mit Blanks auf die Mindestbreite 8     |
|          verlaengert.                                               |
+---------------------------------------------------------------------+

Dieses Programm gibt zahlreiche printf-Befehle aus (so, wie sie auch
in einem C-Programm stehen koennen) und jeweils danch die Ausgabe,
die der printf-Befehl erzeugt.

Hinweis: Der Gnu-C-Compiler gcc kennt die Formatsequenzen
%a, %A und %n nicht (auch wenn man mit -std=c99 compiliert).
------------------------------------------------------------------------ */
#include <stdio.h>   // E/A in Bytestroeme, printf, sprintf, ...
#include <string.h>  // memcopy, memmove, strcpy, strcat, ...
// ------------------------------------------------------------------------
// Ein paar String-Konstanten:
char const * const DAZ = "\""; // Ein Doppeltes Anfuehrungs Zeichen
char const * const ZeileAusLauterGleichheitsZeichen =
   "====================================="
   "===============1===5===10===15===20===\n";
char const * const ZeileAusLauterMinusZeichen =
   "-------------------------------------"
   "---------------1---5---10---15---20---\n";

// Die Zahlen      1...5...10... am Ende der Zeilen sind eine Art
// "Spaltenlineal". Sie sollen dem Benuzter helfen, die Laenge der
// Ausgaben der printf-Befehle zu ermitteln.

int fi, di; // Fuer for-Schleifen
// ------------------------------------------------------------------------
void gibAusI(char * Formate[], int lenFormate,
             int      Daten[], int lenDaten)
{
   // Gibt Werte des Typs int formatiert aus.
   char ForMitDaz[100];
   printf(ZeileAusLauterGleichheitsZeichen);
   printf("%-47s |\n\n", "Typ: int");
   for (fi=0; fi<lenFormate; fi++) {
      char * Format = Formate[fi];
      strcpy(ForMitDaz, DAZ);
      strcat(ForMitDaz, Format);
      strcat(ForMitDaz, DAZ);
      if (fi>0) printf(ZeileAusLauterMinusZeichen);
      for (di=0; di<lenDaten; di++) {

         int Datum = Daten[di];

         printf("printf(%-14s, %24d): ", ForMitDaz, Datum);
         printf(Format, Datum);
         printf("\n");
      }
   }
} // gibAusI
// ------------------------------------------------------------------------
void gibAusU(char *       Formate[], int lenFormate,
             unsigned int   Daten[], int lenDaten)
{
   // Gibt Werte des Typs unsigned int formatiert aus.
   char ForMitDaz[100];
   printf(ZeileAusLauterGleichheitsZeichen);
   printf("%-47s |\n\n", "Typ: unsigned int");
   for (fi=0; fi<lenFormate; fi++) {
      char * Format = Formate[fi];
      strcpy(ForMitDaz, DAZ);
      strcat(ForMitDaz, Format);
      strcat(ForMitDaz, DAZ);
      if (fi>0) printf(ZeileAusLauterMinusZeichen);
      for (di=0; di<lenDaten; di++) {
         unsigned int Datum = Daten[di];

         printf("printf(%-14s, %24d): ", ForMitDaz, Datum);
         printf(Format, Datum);
         printf("\n");
      }
   }
} // gibAusU
// ------------------------------------------------------------------------
void gibAusD(char * Formate[], int lenFormate,
             double   Daten[], int lenDaten)
{
   // Gibt Werte des Typs double formatiert aus.
   char ForMitDaz[100];
   printf(ZeileAusLauterGleichheitsZeichen);
   printf("%-47s |\n\n", "Typ: double");
   for (fi=0; fi<lenFormate; fi++) {
      char * Format = Formate[fi];
      strcpy(ForMitDaz, DAZ);
      strcat(ForMitDaz, Format);
      strcat(ForMitDaz, DAZ);
      if (fi>0) printf(ZeileAusLauterMinusZeichen);
      for (di=0; di<lenDaten; di++) {
         double Datum = Daten[di];

         printf("printf(%-14s, %24.18g): ", ForMitDaz, Datum);
         printf(Format, Datum);
         printf("\n");
      }
   }
} // gibAusD
// ------------------------------------------------------------------------
void gibAusC(char * Formate[], int lenFormate,
             char     Daten[], int lenDaten)
{
   // Gibt Werte des Typs char formatiert aus.
   char ForMitDaz[100];
   printf(ZeileAusLauterGleichheitsZeichen);
   printf("%-47s |\n\n", "Typ: char");
   for (fi=0; fi<lenFormate; fi++) {
      char * Format = Formate[fi];
      strcpy(ForMitDaz, DAZ);
      strcat(ForMitDaz, Format);
      strcat(ForMitDaz, DAZ);
      if (fi>0) printf(ZeileAusLauterMinusZeichen);
      for (di=0; di<lenDaten; di++) {
         char Datum = Daten[di];

         printf("printf(%-14s, %21s'%c'): ", ForMitDaz, " ", Datum);
         printf(Format, Datum);
         printf("\n");
      }
   }
} // gibAusC
// ------------------------------------------------------------------------
void gibAusS(char * Formate[], int lenFormate,
             char *   Daten[], int lenDaten)
{
   // Gibt Werte des Typs char * (String)  formatiert aus.
   char ForMitDaz[100];
   char DatMitDaz[100] = "";
   printf(ZeileAusLauterGleichheitsZeichen);
   printf("%-47s |\n\n", "Typ: char * (String)");
   for (fi=0; fi<lenFormate; fi++) {
      char * Format = Formate[fi];
      strcpy(ForMitDaz, DAZ);
      strcat(ForMitDaz, Format);
      strcat(ForMitDaz, DAZ);
      if (fi>0) printf(ZeileAusLauterMinusZeichen);
      for (di=0; di<lenDaten; di++) {
         char * Datum = Daten[di];
         strcpy(DatMitDaz, "\"");
         strcat(DatMitDaz, Datum);
         strcat(DatMitDaz, "\"");
         printf("printf(%-14s, %24s): ", ForMitDaz, Datum);
         printf(Format, Datum);
         printf("\n");
      }
   }
} // gibAusS
// ------------------------------------------------------------------------
void gibAusA(char * Formate[], int lenFormate,
             void *   Daten[], int lenDaten)
{
   // Gibt Werte des Typs void * (Adressen) formatiert aus.
   char ForMitDaz[100];
   printf(ZeileAusLauterGleichheitsZeichen);
   printf("%-47s |\n\n", "Typ: void * (Adresse)");
   for (fi=0; fi<lenFormate; fi++) {
      char * Format = Formate[fi];
      strcpy(ForMitDaz, DAZ);
      strcat(ForMitDaz, Format);
      strcat(ForMitDaz, DAZ);
      if (fi>0) printf(ZeileAusLauterMinusZeichen);
      for (di=0; di<lenDaten; di++) {
         void * Datum = Daten[di];

         printf("printf(%-14s, %24d): ", ForMitDaz, Datum);
         printf(Format, Datum);
         printf("\n");
      }
   }
} // gibAusA
// ------------------------------------------------------------------------
int main() {

   // ------------------------------------------------------------------------
   // Reihungen (arrays) von Daten, die formatiert und ausgegeben werden
   // sollen. Hier kann man weitere Daten einfuegen:

   int            intR[] = {123, 0, -1, -1234567890};
   short        shortR[] = {123, 0, -1, -123};
   long          longR[] = {123, 0, -1, -1234567890};

   unsigned
   int           uintR[] = {161, 0, 1234567890};
   unsigned
   short       ushortR[] = {123, 0, 65321};
   unsigned
   long         ulongR[] = {123, 0, 1234567890};

   double      doubleR[] = {     +1.5,  +12.345,              +0.0,
                             +1.0E+15, +1.0E-20, +12345.6789012345,
                                 -1.5,  -12.345,              -0.0,
                             -1.0E+15, -1.0E-20, -12345.6789012345,
                           };

   char          charR[] = {'A', 'z', '', ''};

   char *      stringR[] = {"Hallo!", "X", "", "([{!?.;:}])", "12345"};

   void *      adressR[] = {(void *) gibAusI, (void *) main, (void *) intR};
   // ------------------------------------------------------------------------
   // Formatstrings fuer printf-Anweisungen
   // Hier kann man weitere Formatstrings einfuegen:

   char *    intFormat[] = {"->%d<-",      "->%+7d<-",    "->%-+15d<-",
                            "->%0 15d<-",  "->%8.5d<-"
                           };

   char *   uintFormat[] = {"->%u<-",      "->%7u<-",     "->%-15u<-",
                            "->%015u<-",   "->%6X<-",     "->%#6x<-",
                            "->%o<-",      "->%#o<-"
                           };

   char * doubleFormat[] = {"->%f<-",      "->% e<-",     "->%+g<-",
                            "->%+20.2f<-", "->% 11.3E<-", "->%+.4g<-",
                            "->%+10.4G<-", "->%8a<-",     "->%8A<-"
                           };

   char *   charFormat[] = {"->%c<-",      "->%3c<-",     "->%-3c<-"};

   char * stringFormat[] = {"->%s<-",      "->%20s<-",    "->%-20.4s<-",
                            "->%020s<-",   "->%0-20s<-",
                           };

   char * adressFormat[] = {"->%p<-",      "->%10p<-",    "->%-10p<-"};

   // ------------------------------------------------------------------------
   // Der Gnu C-Compiler kennt die Formatsequenz %n nicht:
   int    anzahl = 999;
   char * text   = "Hallo Sonja!";
   // ------------------------------------------------------------------------
   printf("printf01: Jetzt geht es los!\n");
   printf(ZeileAusLauterGleichheitsZeichen);
   printf("printf-Anweisung:                               |   Ausgabe\n");


   // Aufrufe der Funktionen gibAusX:
   gibAusI(   intFormat, sizeof(   intFormat)/ sizeof(   intFormat[0]),
              intR,      sizeof(   intR)     / sizeof(   intR     [0])
          );

   gibAusU(  uintFormat, sizeof(  uintFormat)/ sizeof(  uintFormat[0]),
             uintR,      sizeof(  uintR)     / sizeof(  uintR     [0])
          );

   gibAusD(doubleFormat, sizeof(doubleFormat)/ sizeof(doubleFormat[0]),
           doubleR,      sizeof(doubleR)     / sizeof(doubleR     [0])
          );

   gibAusC(  charFormat, sizeof(  charFormat)/ sizeof(  charFormat[0]),
             charR,      sizeof(  charR)     / sizeof(  charR     [0])
          );

   gibAusS(stringFormat, sizeof(stringFormat)/ sizeof(stringFormat[0]),
           stringR,      sizeof(stringR)     / sizeof(stringR     [0])
          );

   gibAusA(adressFormat, sizeof(adressFormat)/ sizeof(adressFormat[0]),
           adressR,      sizeof(adressR)     / sizeof(adressR     [0])
          );
   printf(ZeileAusLauterGleichheitsZeichen);

   printf("printf(\"%%s,%%n %%d\\n\", text, &anzahl, anzahl);\n");
   printf(" Ist-Ausgabe: %s,%n %d\n", text, &anzahl, anzahl);
   printf("Soll-Ausgabe: %s, %d\n",   text, strlen(text));
   printf("-------------------------------------------\n");
   printf("printf01: Das war's erstmal!\n");
   return 0;
} // main
/* ------------------------------------------------------------------------
Ausgabe dieses Programms:

printf01: Jetzt geht es los!
====================================================1===5===10===15===20===
printf-Anweisung:                               |   Ausgabe
====================================================1===5===10===15===20===
Typ: int                                        |

printf("->%d<-"      ,                      123): ->123<-
printf("->%d<-"      ,                        0): ->0<-
printf("->%d<-"      ,                       -1): ->-1<-
printf("->%d<-"      ,              -1234567890): ->-1234567890<-
----------------------------------------------------1---5---10---15---20---
printf("->%+7d<-"    ,                      123): ->   +123<-
printf("->%+7d<-"    ,                        0): ->     +0<-
printf("->%+7d<-"    ,                       -1): ->     -1<-
printf("->%+7d<-"    ,              -1234567890): ->-1234567890<-
----------------------------------------------------1---5---10---15---20---
printf("->%-+15d<-"  ,                      123): ->+123           <-
printf("->%-+15d<-"  ,                        0): ->+0             <-
printf("->%-+15d<-"  ,                       -1): ->-1             <-
printf("->%-+15d<-"  ,              -1234567890): ->-1234567890    <-
----------------------------------------------------1---5---10---15---20---
printf("->%0 15d<-"  ,                      123): -> 00000000000123<-
printf("->%0 15d<-"  ,                        0): -> 00000000000000<-
printf("->%0 15d<-"  ,                       -1): ->-00000000000001<-
printf("->%0 15d<-"  ,              -1234567890): ->-00001234567890<-
----------------------------------------------------1---5---10---15---20---
printf("->%8.5d<-"   ,                      123): ->   00123<-
printf("->%8.5d<-"   ,                        0): ->   00000<-
printf("->%8.5d<-"   ,                       -1): ->  -00001<-
printf("->%8.5d<-"   ,              -1234567890): ->-1234567890<-
====================================================1===5===10===15===20===
Typ: unsigned int                               |

printf("->%u<-"      ,                      161): ->161<-
printf("->%u<-"      ,                        0): ->0<-
printf("->%u<-"      ,               1234567890): ->1234567890<-
----------------------------------------------------1---5---10---15---20---
printf("->%7u<-"     ,                      161): ->    161<-
printf("->%7u<-"     ,                        0): ->      0<-
printf("->%7u<-"     ,               1234567890): ->1234567890<-
----------------------------------------------------1---5---10---15---20---
printf("->%-15u<-"   ,                      161): ->161            <-
printf("->%-15u<-"   ,                        0): ->0              <-
printf("->%-15u<-"   ,               1234567890): ->1234567890     <-
----------------------------------------------------1---5---10---15---20---
printf("->%015u<-"   ,                      161): ->000000000000161<-
printf("->%015u<-"   ,                        0): ->000000000000000<-
printf("->%015u<-"   ,               1234567890): ->000001234567890<-
----------------------------------------------------1---5---10---15---20---
printf("->%6X<-"     ,                      161): ->    A1<-
printf("->%6X<-"     ,                        0): ->     0<-
printf("->%6X<-"     ,               1234567890): ->499602D2<-
----------------------------------------------------1---5---10---15---20---
printf("->%#6x<-"    ,                      161): ->  0xa1<-
printf("->%#6x<-"    ,                        0): ->     0<-
printf("->%#6x<-"    ,               1234567890): ->0x499602d2<-
----------------------------------------------------1---5---10---15---20---
printf("->%o<-"      ,                      161): ->241<-
printf("->%o<-"      ,                        0): ->0<-
printf("->%o<-"      ,               1234567890): ->11145401322<-
----------------------------------------------------1---5---10---15---20---
printf("->%#o<-"     ,                      161): ->0241<-
printf("->%#o<-"     ,                        0): ->0<-
printf("->%#o<-"     ,               1234567890): ->011145401322<-
====================================================1===5===10===15===20===
Typ: double                                     |

printf("->%f<-"      ,                      1.5): ->1.500000<-
printf("->%f<-"      ,      12.3450000000000006): ->12.345000<-
printf("->%f<-"      ,                        0): ->0.000000<-
printf("->%f<-"      ,         1000000000000000): ->1000000000000000.000000<-
printf("->%f<-"      ,  9.99999999999999945e-21): ->0.000000<-
printf("->%f<-"      ,      12345.6789012344998): ->12345.678901<-
printf("->%f<-"      ,                     -1.5): ->-1.500000<-
printf("->%f<-"      ,     -12.3450000000000006): ->-12.345000<-
printf("->%f<-"      ,                       -0): ->-0.000000<-
printf("->%f<-"      ,        -1000000000000000): ->-1000000000000000.000000<-
printf("->%f<-"      , -9.99999999999999945e-21): ->-0.000000<-
printf("->%f<-"      ,     -12345.6789012344998): ->-12345.678901<-
----------------------------------------------------1---5---10---15---20---
printf("->% e<-"     ,                      1.5): -> 1.500000e+00<-
printf("->% e<-"     ,      12.3450000000000006): -> 1.234500e+01<-
printf("->% e<-"     ,                        0): -> 0.000000e+00<-
printf("->% e<-"     ,         1000000000000000): -> 1.000000e+15<-
printf("->% e<-"     ,  9.99999999999999945e-21): -> 1.000000e-20<-
printf("->% e<-"     ,      12345.6789012344998): -> 1.234568e+04<-
printf("->% e<-"     ,                     -1.5): ->-1.500000e+00<-
printf("->% e<-"     ,     -12.3450000000000006): ->-1.234500e+01<-
printf("->% e<-"     ,                       -0): ->-0.000000e+00<-
printf("->% e<-"     ,        -1000000000000000): ->-1.000000e+15<-
printf("->% e<-"     , -9.99999999999999945e-21): ->-1.000000e-20<-
printf("->% e<-"     ,     -12345.6789012344998): ->-1.234568e+04<-
----------------------------------------------------1---5---10---15---20---
printf("->%+g<-"     ,                      1.5): ->+1.5<-
printf("->%+g<-"     ,      12.3450000000000006): ->+12.345<-
printf("->%+g<-"     ,                        0): ->+0<-
printf("->%+g<-"     ,         1000000000000000): ->+1e+15<-
printf("->%+g<-"     ,  9.99999999999999945e-21): ->+1e-20<-
printf("->%+g<-"     ,      12345.6789012344998): ->+12345.7<-
printf("->%+g<-"     ,                     -1.5): ->-1.5<-
printf("->%+g<-"     ,     -12.3450000000000006): ->-12.345<-
printf("->%+g<-"     ,                       -0): ->-0<-
printf("->%+g<-"     ,        -1000000000000000): ->-1e+15<-
printf("->%+g<-"     , -9.99999999999999945e-21): ->-1e-20<-
printf("->%+g<-"     ,     -12345.6789012344998): ->-12345.7<-
----------------------------------------------------1---5---10---15---20---
printf("->%+20.2f<-" ,                      1.5): ->               +1.50<-
printf("->%+20.2f<-" ,      12.3450000000000006): ->              +12.35<-
printf("->%+20.2f<-" ,                        0): ->               +0.00<-
printf("->%+20.2f<-" ,         1000000000000000): ->+1000000000000000.00<-
printf("->%+20.2f<-" ,  9.99999999999999945e-21): ->               +0.00<-
printf("->%+20.2f<-" ,      12345.6789012344998): ->           +12345.68<-
printf("->%+20.2f<-" ,                     -1.5): ->               -1.50<-
printf("->%+20.2f<-" ,     -12.3450000000000006): ->              -12.35<-
printf("->%+20.2f<-" ,                       -0): ->               -0.00<-
printf("->%+20.2f<-" ,        -1000000000000000): ->-1000000000000000.00<-
printf("->%+20.2f<-" , -9.99999999999999945e-21): ->               -0.00<-
printf("->%+20.2f<-" ,     -12345.6789012344998): ->           -12345.68<-
----------------------------------------------------1---5---10---15---20---
printf("->% 11.3E<-" ,                      1.5): ->  1.500E+00<-
printf("->% 11.3E<-" ,      12.3450000000000006): ->  1.235E+01<-
printf("->% 11.3E<-" ,                        0): ->  0.000E+00<-
printf("->% 11.3E<-" ,         1000000000000000): ->  1.000E+15<-
printf("->% 11.3E<-" ,  9.99999999999999945e-21): ->  1.000E-20<-
printf("->% 11.3E<-" ,      12345.6789012344998): ->  1.235E+04<-
printf("->% 11.3E<-" ,                     -1.5): -> -1.500E+00<-
printf("->% 11.3E<-" ,     -12.3450000000000006): -> -1.235E+01<-
printf("->% 11.3E<-" ,                       -0): -> -0.000E+00<-
printf("->% 11.3E<-" ,        -1000000000000000): -> -1.000E+15<-
printf("->% 11.3E<-" , -9.99999999999999945e-21): -> -1.000E-20<-
printf("->% 11.3E<-" ,     -12345.6789012344998): -> -1.235E+04<-
----------------------------------------------------1---5---10---15---20---
printf("->%+.4g<-"   ,                      1.5): ->+1.5<-
printf("->%+.4g<-"   ,      12.3450000000000006): ->+12.35<-
printf("->%+.4g<-"   ,                        0): ->+0<-
printf("->%+.4g<-"   ,         1000000000000000): ->+1e+15<-
printf("->%+.4g<-"   ,  9.99999999999999945e-21): ->+1e-20<-
printf("->%+.4g<-"   ,      12345.6789012344998): ->+1.235e+04<-
printf("->%+.4g<-"   ,                     -1.5): ->-1.5<-
printf("->%+.4g<-"   ,     -12.3450000000000006): ->-12.35<-
printf("->%+.4g<-"   ,                       -0): ->-0<-
printf("->%+.4g<-"   ,        -1000000000000000): ->-1e+15<-
printf("->%+.4g<-"   , -9.99999999999999945e-21): ->-1e-20<-
printf("->%+.4g<-"   ,     -12345.6789012344998): ->-1.235e+04<-
----------------------------------------------------1---5---10---15---20---
printf("->%+10.4G<-" ,                      1.5): ->      +1.5<-
printf("->%+10.4G<-" ,      12.3450000000000006): ->    +12.35<-
printf("->%+10.4G<-" ,                        0): ->        +0<-
printf("->%+10.4G<-" ,         1000000000000000): ->    +1E+15<-
printf("->%+10.4G<-" ,  9.99999999999999945e-21): ->    +1E-20<-
printf("->%+10.4G<-" ,      12345.6789012344998): ->+1.235E+04<-
printf("->%+10.4G<-" ,                     -1.5): ->      -1.5<-
printf("->%+10.4G<-" ,     -12.3450000000000006): ->    -12.35<-
printf("->%+10.4G<-" ,                       -0): ->        -0<-
printf("->%+10.4G<-" ,        -1000000000000000): ->    -1E+15<-
printf("->%+10.4G<-" , -9.99999999999999945e-21): ->    -1E-20<-
printf("->%+10.4G<-" ,     -12345.6789012344998): ->-1.235E+04<-
----------------------------------------------------1---5---10---15---20---
printf("->%8a<-"     ,                      1.5): ->       a<-
printf("->%8a<-"     ,      12.3450000000000006): ->       a<-
printf("->%8a<-"     ,                        0): ->       a<-
printf("->%8a<-"     ,         1000000000000000): ->       a<-
printf("->%8a<-"     ,  9.99999999999999945e-21): ->       a<-
printf("->%8a<-"     ,      12345.6789012344998): ->       a<-
printf("->%8a<-"     ,                     -1.5): ->       a<-
printf("->%8a<-"     ,     -12.3450000000000006): ->       a<-
printf("->%8a<-"     ,                       -0): ->       a<-
printf("->%8a<-"     ,        -1000000000000000): ->       a<-
printf("->%8a<-"     , -9.99999999999999945e-21): ->       a<-
printf("->%8a<-"     ,     -12345.6789012344998): ->       a<-
----------------------------------------------------1---5---10---15---20---
printf("->%8A<-"     ,                      1.5): ->       A<-
printf("->%8A<-"     ,      12.3450000000000006): ->       A<-
printf("->%8A<-"     ,                        0): ->       A<-
printf("->%8A<-"     ,         1000000000000000): ->       A<-
printf("->%8A<-"     ,  9.99999999999999945e-21): ->       A<-
printf("->%8A<-"     ,      12345.6789012344998): ->       A<-
printf("->%8A<-"     ,                     -1.5): ->       A<-
printf("->%8A<-"     ,     -12.3450000000000006): ->       A<-
printf("->%8A<-"     ,                       -0): ->       A<-
printf("->%8A<-"     ,        -1000000000000000): ->       A<-
printf("->%8A<-"     , -9.99999999999999945e-21): ->       A<-
printf("->%8A<-"     ,     -12345.6789012344998): ->       A<-
====================================================1===5===10===15===20===
Typ: char                                       |

printf("->%c<-"      ,                      'A'): ->A<-
printf("->%c<-"      ,                      'z'): ->z<-
printf("->%c<-"      ,                      ''): -><-
printf("->%c<-"      ,                      ''): -><-
----------------------------------------------------1---5---10---15---20---
printf("->%3c<-"     ,                      'A'): ->  A<-
printf("->%3c<-"     ,                      'z'): ->  z<-
printf("->%3c<-"     ,                      ''): ->  <-
printf("->%3c<-"     ,                      ''): ->  <-
----------------------------------------------------1---5---10---15---20---
printf("->%-3c<-"    ,                      'A'): ->A  <-
printf("->%-3c<-"    ,                      'z'): ->z  <-
printf("->%-3c<-"    ,                      ''): ->  <-
printf("->%-3c<-"    ,                      ''): ->  <-
====================================================1===5===10===15===20===
Typ: char * (String)                            |

printf("->%s<-"      ,                   Hallo!): ->Hallo!<-
printf("->%s<-"      ,                        X): ->X<-
printf("->%s<-"      ,                         ): -><-
printf("->%s<-"      ,              ([{!?.;:}])): ->([{!?.;:}])<-
printf("->%s<-"      ,                    12345): ->12345<-
----------------------------------------------------1---5---10---15---20---
printf("->%20s<-"    ,                   Hallo!): ->              Hallo!<-
printf("->%20s<-"    ,                        X): ->                   X<-
printf("->%20s<-"    ,                         ): ->                    <-
printf("->%20s<-"    ,              ([{!?.;:}])): ->         ([{!?.;:}])<-
printf("->%20s<-"    ,                    12345): ->               12345<-
----------------------------------------------------1---5---10---15---20---
printf("->%-20.4s<-" ,                   Hallo!): ->Hall                <-
printf("->%-20.4s<-" ,                        X): ->X                   <-
printf("->%-20.4s<-" ,                         ): ->                    <-
printf("->%-20.4s<-" ,              ([{!?.;:}])): ->([{!                <-
printf("->%-20.4s<-" ,                    12345): ->1234                <-
----------------------------------------------------1---5---10---15---20---
printf("->%020s<-"   ,                   Hallo!): ->00000000000000Hallo!<-
printf("->%020s<-"   ,                        X): ->0000000000000000000X<-
printf("->%020s<-"   ,                         ): ->00000000000000000000<-
printf("->%020s<-"   ,              ([{!?.;:}])): ->000000000([{!?.;:}])<-
printf("->%020s<-"   ,                    12345): ->00000000000000012345<-
----------------------------------------------------1---5---10---15---20---
printf("->%0-20s<-"  ,                   Hallo!): ->Hallo!              <-
printf("->%0-20s<-"  ,                        X): ->X                   <-
printf("->%0-20s<-"  ,                         ): ->                    <-
printf("->%0-20s<-"  ,              ([{!?.;:}])): ->([{!?.;:}])         <-
printf("->%0-20s<-"  ,                    12345): ->12345               <-
====================================================1===5===10===15===20===
Typ: void * (Adresse)                           |

printf("->%p<-"      ,                  4198480): ->0x401050<-
printf("->%p<-"      ,                  4200328): ->0x401788<-
printf("->%p<-"      ,                  2280624): ->0x22ccb0<-
----------------------------------------------------1---5---10---15---20---
printf("->%10p<-"    ,                  4198480): ->  0x401050<-
printf("->%10p<-"    ,                  4200328): ->  0x401788<-
printf("->%10p<-"    ,                  2280624): ->  0x22ccb0<-
----------------------------------------------------1---5---10---15---20---
printf("->%-10p<-"   ,                  4198480): ->0x401050  <-
printf("->%-10p<-"   ,                  4200328): ->0x401788  <-
printf("->%-10p<-"   ,                  2280624): ->0x22ccb0  <-
====================================================1===5===10===15===20===
printf("%s,%n %d\n", text, &anzahl, anzahl);
 Ist-Ausgabe: Hallo Sonja!, 999
Soll-Ausgabe: Hallo Sonja!, 12
-------------------------------------------
printf01: Das war's erstmal!

Prozess erfolgreich beendet.
------------------------------------------------------------------------ */
