// Datei printfDouble01.c
/* ------------------------------------------------------------------------
Dieses Programm liest wiederholt eine Formatsequenz ein (z.B. %f oder
%+#20.10g etc.) und gibt ein paar double-Werte mit ihr aus.
Mit diesem Programm kann man durch "Ausprobieren" die zahlreichen
Moeglichkeiten erforschen, die die printf-Funktion zum Formatieren
von double-Werten bietet.
---------------------------------------------------------------------------
Erlaeuterungen und Fachbegriffe:

Ein typischer Aufruf der printf-Funktion sieht z.B. so aus:

printf("Anzahl: %2d, Gewicht: %4.2f, Gesamt: %6.2f\n", anz, g, anz*g);

Der erste Parameter von printf muss immer ein String sein (d.h. zum
Typ char * gehoeren) und wird als FORMATSTRING bezeichnet.

Der Formatstring kann (null, eine oder mehrere) FORMATSEQUENZEN enthalten
Das sind Zeichenfolgen, die mit einem Prozentzeichen % beginnen und mit
einem speziellen Umwandlungsbuchstaben enden (z.B. mit d oder f ode X etc.)
Im Beispiel oben enthaelt der Formatstring die 3 Formatsequenzen
%2d, %4.2f und %6.2f.

Fuer jede Formatsequenz im Formatstring muss man (nach dem Formatstring)
einen ZUSAETZLICHEN PARAMETER angeben. Im Beispiel wurden die 3 zusaetz-
lichen Parameter anz, g und anz*g angegeben. Zur 1., 2., 3., ... Format-
sequenz gehoert also der 1., 2., 3., ... zusaetzliche Parameter.

Der Typ eines zusaetzlichen Parameters muss zum Umwandlungsbuchstaben
"seiner Formatsequenz" passen: Z.B. passt der Typ int zu den Umwandlungs-
buchstaben d, o, x und X. Der Typ double passt zu den Umwandlungsbuchstaben
f, e und g etc. Im Beispiel sollte die Variable anz zum Typ int und die
Variable g zum Typ double gehoeren (der Ausdruck anz*g gehoert dann auto-
matisch auch zum Typ double).

Die Ausfuehrung eines printf-Aufrufs erfolgt in 3 Schritten:
1. Jeder zusaetzliche Parameter wird entsprechend "seiner Formatsequenz"
   formatiert (d.h. "auf spezielle Weise in einen String umgewandelt").
2. In einer Kopie des Formatstrings wird jede Formatsequenz durch den
   entsprechenden (formatierten) zusaetzlichen Parameter ersetzt.
3. Die Kopie des Formatstrings (mit den formatierten zusaetzlichen
   Parametern darin) wird zur Standardausgabe ausgegeben.

Ist anz eine int-Variable mit dem Wert 5 und g eine double-Variable mit
dem Wert 3.1, dann produziert der obige printf-Befehl folgende Ausgabe:

Anzahl:  5, Gewicht: 3.10, Gesamt:  15.50
---------------------------------------------------------------------------
In diesem Programm wird wiederholt eine printf-Anweisung ausgefuehrt,
deren Formatstring nur EINE Formatsequenz enthaelt. Diese Formatsequenz
kann der Benutzer eingeben. Sie sollte zum Formatieren von double-Werten
geeignet sein, d.h. mit einem der Umwandlungsbuchstaben f, e, g, a oder A
enden.
Achtung: Der Gnu-C-Ausfuehrer kennt die Umwandlungsbuchtagen a und A nicht.
------------------------------------------------------------------------ */
#include <stdio.h>   // E/A in Bytestroeme, printf, sprintf, ...
#include <string.h>  // memcopy, memmove, strcpy, strcat, ...

// Mit printf ausgegeben wird ein Wert des Typs TYP:
typedef double TYP;
// Maximale Laenge der Formatstrings formatM und formatO:
#define MAXLEN 100
// Weil einige C-Ausfuehrer stdbool.h und true immer noch nicht kennen
// (z.B. cl von Microsoft und icl von Intel):
#define true 1

int main() {
   // Den Formatstring fuer die "printf-Anweisung um die es hier geht"
   // brauchen wir in 2 Varianten:
   // MIT  doppelten Anfuehrungszeichen zum Ausgeben der Anweisung selbst
   // OHNE doppelte  Anfuehrungszeichen zum Ausgeben ihrer Ausgabe
   char formatM[MAXLEN];  // Formatstring MIT  doppelten Anfuehrungszeichen
   char formatO[MAXLEN];  // Formatstring OHNE doppelte  Anfuehrungszeichen

   // Puffer zum Einlesen von Daten, Muell (newline-Zeichen) und
   // einem einzelnen Zeichen ('q' fuer quit):
   char daten[MAXLEN];
   int  einZeichen;
   int  i;

   // int-Werte, die mit jeder eingelesenen Formatsequenz formatiert und
   // ausgegeben werden:
   TYP tab[] = {+2.5, +12.345678, +123456.78, +1.0E-5, +0.0,
                -2.5, -12.345678, -123456.78, -1.0E-5, -0.0
               };
   int len   = sizeof(tab)/sizeof(tab[0]);

   printf("printfDouble01: Jetzt geht es los!\n");

   while (true) {
      printf("------------------------------------------------------  \n");
      printf("Bitte geben Sie eine Formatsequenz FS fuer das Forma-   \n");
      printf("tieren und Ausgeben eines double-Wertes ein (oder q zum \n");
      printf("Beenden). Moegliche Werte fuer FS sind z.B.:          \n\n");
      printf("%%f oder %%9f oder %%+.2f oder %% 10.2f  oder           \n");
      printf("%%e oder %%4e oder %%7.2e oder %%+g oder %%#.0g etc.  \n\n");
      printf("FS = ");

      // Soll das Programm beendet werden?
      einZeichen = getc(stdin);     // 1 Zeichen aus dem Puffer holen
      printf("\n");
      if (einZeichen == 'q') {
         break;
      } else {
         ungetc(einZeichen, stdin); // 1 Zeichen in den Puffer zuruecklegen
      } // if

      // Eine Formatsequenz einlesen (als ganze Zeile):
      fgets(daten, MAXLEN-8, stdin);  // -8 fuer doppelte Anfuehrungs-
                                      // zeichen und Pfeile, s.u.
      daten[strlen(daten)-1] = '\0';  // Das newline-Zeichen loeschen

      // Die eingelesene Formatsequenz vorn und hinten um Pfeile ergaenzen,
      // damit man spaeter Blanks etc. am Ende der Ausgabe "sehen kann":
      strcpy(formatM, "\"-->"); // 4 Zeichen
      strcat(formatM, daten);
      strcat(formatM, "<--\""); // plus 4 Zeichen sind 8 Zeichen

      strcpy(formatO, "-->");   // 3 Zeichen
      strcat(formatO, daten);
      strcat(formatO, "<--");   // plus 3 Zeichen sind 6 Zeichen

      // Fuer jede Zahl in tab eine printf-Anweisung ausgeben (so wie sie
      // in einem C-Programm aussehen koennte) und dahinter die Ausgabe,
      // die diese Anweisung produziert:
      printf("%43s| %-s\n", "Eine printf-Anweisung:", "Ihre Ausgabe:");
      for (i=0; i<len; i++) {
         TYP var = tab[i];
         printf("printf(%-18s, %15.7f); ", formatM, var);
         printf(formatO, var);
         printf("\n");
      } // for i

   } // while (true)

   printf("----------------------------------------------------------\n");
   printf("printfDouble01: Das war's erstmal!\n");
   return 0;
} // main
/* ------------------------------------------------------------------------
Ein Dialog mit diesem Programm:

printfDouble01: Jetzt geht es los!
------------------------------------------------------
Bitte geben Sie eine Formatsequenz FS fuer das Forma-
tieren und Ausgeben eines double-Wertes ein (oder q zum
Beenden). Moegliche Werte fuer FS sind z.B.:

%f oder %9f oder %+.2f oder % 10.2f  oder
%e oder %4e oder %7.2e oder %+g oder %#.0g etc.

FS = % 12.2f

                     Eine printf-Anweisung:| Ihre Ausgabe:
printf("-->% 12.2f<--"   ,       2.5000000); -->        2.50<--
printf("-->% 12.2f<--"   ,      12.3456780); -->       12.35<--
printf("-->% 12.2f<--"   ,  123456.7800000); -->   123456.78<--
printf("-->% 12.2f<--"   ,       0.0000100); -->        0.00<--
printf("-->% 12.2f<--"   ,       0.0000000); -->        0.00<--
printf("-->% 12.2f<--"   ,      -2.5000000); -->       -2.50<--
printf("-->% 12.2f<--"   ,     -12.3456780); -->      -12.35<--
printf("-->% 12.2f<--"   , -123456.7800000); -->  -123456.78<--
printf("-->% 12.2f<--"   ,      -0.0000100); -->       -0.00<--
printf("-->% 12.2f<--"   ,      -0.0000000); -->       -0.00<--
------------------------------------------------------
Bitte geben Sie eine Formatsequenz FS fuer das Forma-
tieren und Ausgeben eines double-Wertes ein (oder q zum
Beenden). Moegliche Werte fuer FS sind z.B.:

%f oder %9f oder %+.2f oder % 10.2f  oder
%e oder %4e oder %7.2e oder %+g oder %#.0g etc.

FS = %+9.2e

                     Eine printf-Anweisung:| Ihre Ausgabe:
printf("-->%+9.2e<--"    ,       2.5000000); -->+2.50e+00<--
printf("-->%+9.2e<--"    ,      12.3456780); -->+1.23e+01<--
printf("-->%+9.2e<--"    ,  123456.7800000); -->+1.23e+05<--
printf("-->%+9.2e<--"    ,       0.0000100); -->+1.00e-05<--
printf("-->%+9.2e<--"    ,       0.0000000); -->+0.00e+00<--
printf("-->%+9.2e<--"    ,      -2.5000000); -->-2.50e+00<--
printf("-->%+9.2e<--"    ,     -12.3456780); -->-1.23e+01<--
printf("-->%+9.2e<--"    , -123456.7800000); -->-1.23e+05<--
printf("-->%+9.2e<--"    ,      -0.0000100); -->-1.00e-05<--
printf("-->%+9.2e<--"    ,      -0.0000000); -->-0.00e+00<--
------------------------------------------------------
Bitte geben Sie eine Formatsequenz FS fuer das Forma-
tieren und Ausgeben eines double-Wertes ein (oder q zum
Beenden). Moegliche Werte fuer FS sind z.B.:

%f oder %9f oder %+.2f oder % 10.2f  oder
%e oder %4e oder %7.2e oder %+g oder %#.0g etc.

FS = %#3.1g

                     Eine printf-Anweisung:| Ihre Ausgabe:
printf("-->%#3.1g<--"    ,       2.5000000); --> 2.<--
printf("-->%#3.1g<--"    ,      12.3456780); -->1.e+01<--
printf("-->%#3.1g<--"    ,  123456.7800000); -->1.e+05<--
printf("-->%#3.1g<--"    ,       0.0000100); -->1.e-05<--
printf("-->%#3.1g<--"    ,       0.0000000); --> 0.<--
printf("-->%#3.1g<--"    ,      -2.5000000); -->-2.<--
printf("-->%#3.1g<--"    ,     -12.3456780); -->-1.e+01<--
printf("-->%#3.1g<--"    , -123456.7800000); -->-1.e+05<--
printf("-->%#3.1g<--"    ,      -0.0000100); -->-1.e-05<--
printf("-->%#3.1g<--"    ,      -0.0000000); -->-0.<--
------------------------------------------------------
Bitte geben Sie eine Formatsequenz FS fuer das Forma-
tieren und Ausgeben eines double-Wertes ein (oder q zum
Beenden). Moegliche Werte fuer FS sind z.B.:

%f oder %9f oder %+.2f oder % 10.2f  oder
%e oder %4e oder %7.2e oder %+g oder %#.0g etc.

FS = q

----------------------------------------------------------
printfDouble01: Das war's erstmal!
------------------------------------------------------------------------ */
