// Datei printfInt02.c
/* ------------------------------------------------------------------------
Programm zum Kennenlernen des printf-Befehls.

In einer aeusseren Schleife wird jeweils ein Formatstring fsa fuer einen
printf-Befehl eingelesen.

In einer inneren Schleife wird dann jeweils ein int-Wert eingelesen und
mit einem printf-Befehl und dem Formatstring fsa wieder ausgegeben.
------------------------------------------------------------------------ */
#include <stdio.h>   // E/A in Bytestroeme, printf, sprintf, ...
#include <string.h>  // memcopy, memmove, strcpy, strcat, ...

// Mit printf ausgegeben wird ein Wert des Typs TYP:
typedef int TYP;
// Maximale Laenge des Formatstrings fsa:
#define MAXLEN 100
// Weil einige C-Ausfuehrer stdbool.h und true immer noch nicht kennen
// (z.B. cl von Microsoft und icl von Intel):
#define true 1
// ------------------------------------------------------------------------
int main() {

   // Beispiele fuer Formatstrings, die der Benutzer eingeben kann:
   const char *b1 = "%d";
   const char *b2 = "%5d";
   const char *b3 = "%x";
   const char *b4 = "%-8d";
   const char *b5 = "q zum Beenden";

   // Meldung in der inneren Schleife, typabhaengig:
   const char *text = "Einen int-Wert fuer var ein"
                      "geben (oder q zum Beenden):\n";

   // Formatstring zum Einlesen des auszugebenden Wertes:
   const char *fse = "%d";

   // Zwei Parameter fuer den printf-Befehl
   char fsa  [MAXLEN];  // Formatstring, wird eingelesen
   TYP  var;            // Variable, deren Wert ausgegeben wird

   // Puffer zum Einlesen von Daten, Muell (newline-Zeichen) und
   // einem einzelnen Zeichen ('q' fuer quit):
   char daten[MAXLEN];
   char muell[MAXLEN];
   int  naechstesZeichen;

   printf("printfInt02: Jetzt geht es los!\n");

   // Die aeussere Schleife dient zum Einlesen von Formatstrings:
   while (true) {
      printf("--------------------------------------------------------\n");
      printf("Mit dem folgenden printf-Befehl soll der Werte der      \n");
      printf("Variablen var (vom Typ int) ausgegeben werden:          \n");
      printf("----------------------------------                      \n");
      printf("printf(fsa, var);                                       \n");
      printf("----------------------------------                      \n");
      printf("Bitte geben Sie zuerst einen  Formatstring fsa ein.     \n");
      printf("Moegliche Werte fuer fsa sind z.B.:                     \n");
      printf("%s oder %s oder %s oder %s (oder %s):\n", b1, b2, b3, b4, b5);
      printf("fsa = ");

      // Soll diese aeussere Schleife (und damit das ganze Programm)
      // beendet werden?
      naechstesZeichen = getc(stdin);
      if (naechstesZeichen == 'q') {
         break;
      } else {
         ungetc(naechstesZeichen, stdin);
      } // if

      // Einen Formatstring einlesen (als ganze Zeile):
      fgets(daten, MAXLEN-6, stdin);
      daten[strlen(daten)-1] = '\0';  // Das newline-Zeichen loeschen

      // Den eingelesenen Formatstring vorn und hinten um Pfeile ergaenzen,
      // damit man spaeter ausgegebene Blanks und Tabs "sehen kann":
      strcpy(fsa, "-->"); // 3 Zeichen
      strcat(fsa, daten);
      strcat(fsa, "<--"); // plus 3 Zeichen sind 6 Zeichen

      printf("-----------------------------------\n");
      printf("Ausgefuehrt wird der printf-Befehl:\n");
      printf("printf(\"%s\", var);\n", fsa);
      printf("-----------------------------------\n");
      printf(text);

      // Eine Art "Lineal" ausgeben, damit der Benutzer die Laenge der
      // Ausgabe (und "was steht an welcher Stelle?") herausfinden kann:
      printf("Lineal mit Zeichen-Nrn:    -->12345678901234567890\n");

      // Die innere Schleife dient zum Einlesen von Werten (die dann
      // mit printf und dem Formatstring fsa wieder ausgegeben werden):
      while (true) {

         printf("var = "); // Eingabeaufforderung

         // Soll diese innere Schleife beendet werden?
         naechstesZeichen = getc(stdin);
         if (naechstesZeichen == 'q') {
            fgets(muell, MAXLEN, stdin); // newline-Zeichen lesen
            break;
         } else {
            ungetc(naechstesZeichen, stdin);
         } // if

         // Den Wert einlesen, der gleich mit printf ausgegeben wird:
         scanf(fse, &var);

         printf("Ausgabe von printf:        ");
         printf (fsa, var);
         printf("\n");

         // Der Benutzer hat seine letzte Eingabe mit Return abgeschlossen,
         // aber der scanf-Befehl hat das newline-Zeichen noch NICHT aus
         // dem Eingabepuffer gelesen. Jetzt wird es gelesen:
         fgets(muell, MAXLEN, stdin);

      } // while
   } // while

   printf("--------------------------------------------------------\n");
   printf("printfInt02: Das war's erstmal!\n");
   return 0;
} // main
